<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Setup of configuration variables, constants and file inclusions
 * used by Anno.
 *
 * @author Heikki Aitakangas, Jukka Huhta, Timo Myyrylinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * 
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 * 
 */

/// Sanity check

if (!defined('MOODLE_INTERNAL')) {
    die('Setup failure!');
}

/// Configuration

/**
 * Address of the Annotea server we store data in.
 */
define ('ANNO_ANNOTEA_ADDRESS', 'http://localhost:12916/cgi-bin/annotea');

/**
 * Location where the RDF schema describing the comment categories is stored.
 * Not really implemented yet, just leave as it is.
 */
define ('ANNO_COMMENT_CATEGORY_NS', 'http://www.cs.helsinki.fi/group/mooan/mooan-category-ns');

/**
 * If this is true, E_USER_* type errors will be set to be reported via a call
 * error_reporting(). Anno prints warnings and notices using trigger_error().
 */
define ('ANNO_SET_ERROR_LEVEL', false);


/**
 * The time (in minutes) that an user can edit or delete her comment, 
 * if not already answered.  After this period of time the comments
 * will be 'locked' and only admin/teacher can remove them.
 */
define ('ANNO_COMMENT_REMOVAL_TIME', 30);

/**
 * Maximum amount of overlapping comments that is distinctively visualized.
 */
define ('ANNO_MAXIMUM_OVERLAPPING_VISUALIZATION', 6);

/**
 * Prefix of variables and functions.
 */
define ('ANNO_PREFIX', 'anno_');


/// Constants
/// The user rights constants used by Anno

/**
 * An unlogged-in user or a guest. May only read comments.
 */
define('ANNO_RIGHTS_VISITOR', 0);

/**
 * A logged-in student or a logged-in teacher who does not have teacher rights in the current course.
 * May read comments and make their own comments. May also edit and remove own comments as long as they
 * have not yet been responded to.
 */
define('ANNO_RIGHTS_NORMAL', 1);

/**
 * A logged-in student who is the owner of the current document. Has the same rights as
 * ANNO_RIGHTS_NORMAL and in addition may toggle whether the current document is annotable.
 */
define('ANNO_RIGHTS_ALTER_ANNOTABILITY', 2);

/**
 * A logged-in admin or a logged-in teacher who is a teacher in the current course.
 * May read comments, make their own, toggle annotability and also edit and delete
 * any comment, including those made by other users.
 */
define('ANNO_RIGHTS_ALL', 4);

/// Requires
/**
 * UTF-8-aware versions of the string functions. Needed in most of the .php files.
 */
require_once($CFG->dirroot . '/anno/utf8/utf8.php');
require_once(UTF8 . '/trim.php');
/**
 * Warning: Possible bug still left on using rawurlencode() or strstr(); they are NOT
 * UTF-8 safe. Test with UTF-8 containing URLs.
 */

require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_annotea_lib.php');
require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_documentlib.php');
require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_handle_request.php');
require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_moodle_interface.php');
require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_readcomments.php');
require_once($CFG->dirroot . '/anno/handler_module/' . 'anno_x_pointer_lib.php');

?>
