<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Library of functions interfacing with Anno-database
 *
 * @author  Kirsi Jokisalo, Timo Myyrylinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * 
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 * 
 */


/** 
 * This function checks, if annotating is on for the document.
 *
 * @param  string $documentID URL of the document that the comment is attached to.
 * @return boolean true, if annotating is on, and false if annotating is off.
 */
function anno_get_annotation_status($documentID) {
    // Use the Moodle database connection
    global $db;

    // Check the parameters
    if(!is_string($documentID)) {
        return false;
    }

    // Lowercase and add slashes to the documentID
    $documentID = addslashes(utf8_strtolower($documentID));

    // Query to database
    // anno_annotation_on holds a list of document_ids, that have annotation switched on.
    $query = "SELECT count(\"document_id\") FROM anno_annotation_on WHERE document_id='$documentID'";

    $dbresult = $db->Execute($query);

    // Check the number of matching documentIDs.
    if ($dbresult && $dbresult->fields[0] == 1) {
        return true;
    }

    return false;
}


/** 
 * This function sets annotating on/off for the document.
 *
 * @param  boolean $status true sets annotation on, false sets annotation off.
 * @param  string  $documentID URL of the document where annotation is being switched on/off.
 * @return boolean true, if annotation was successfully switched on/off
 *                 false, if annotation already was in the same status as $status.
 */
function anno_set_annotation_status($status, $documentID) {
    // Use the Moodle database connection
    global $db;

    // Lowercase and add slashes to the documentID
    $documentID = addslashes(utf8_strtolower($documentID));

    // Check wheter to set annotation on or off
    if($status) {
        if(anno_get_annotation_status($documentID)) {
            // annotating already on
            return false;
        } else {
            // setting annotation on
            $query = "INSERT INTO anno_annotation_on (document_id) VALUES('$documentID')";
        }
    } else {
        if(!anno_get_annotation_status($documentID)) {
            // annotating already off
            return false;
        } else {
            //set annotation off
            $query = "DELETE FROM anno_annotation_on WHERE document_id='$documentID'";
        }
    }
    $db->Execute($query);

    return true;
}

?>
