<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Script interfacing with the ui_module (javascript).
 *
 * @author  Kirsi Jokisalo, Esko Lehtonen, Jukka Huhta
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */

/**
 * This script is the interface between javascript and php-functions.
 * It returns a boolean value according to operation success.
 */

/// Inclusions

require_once("../../config.php");
require_once("anno_configuration.php");

/// Error level
if (ANNO_SET_ERROR_LEVEL) {
    error_reporting(error_reporting() | E_USER_NOTICE | E_USER_WARNING | E_USER_ERROR);
}


/// Functions

/**
 * A function for sending the response to the client.
 * Sends the Content-Type header, adds xml declaration
 * and wraps the given content in "response" tags.
 * All of the text is echoed out.
 *
 * @param string $content The content of the response
 */
function anno_do_response($content) {
    header('Content-Type: text/xml');
    echo '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>';
    echo '<response>';
    echo '  ' . $content;
    echo '</response>';
}


/// Script body

// Load the minimum parameters required to start figuring out what the request was and whether we should fulfill it
$user        = anno_get_current_user_id();
$rights      = anno_get_user_rights();
$action      = optional_param('action',      NULL, PARAM_RAW);
$document_id = optional_param('document_id', '',   PARAM_RAW);

/** Check user rights */
// UI shouldn't let visitors make any operations that call this script. But if they do, whine about it
if ($rights == ANNO_RIGHTS_VISITOR) {
    header("HTTP/1.1 400 BAD REQUEST", true, 400);
    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kyttjll ei ole oikeuksia kytt tt toimintoa.').'</errormessage>';
}

// Verify the basic parameters
else if ($action == NULL) {
    header("HTTP/1.1 400 BAD REQUEST", true, 400); // no action specified - nothing to do
    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: Mitn toimintoa ei ole valittu').'</errormessage>';
}
else if ($document_id == '') {
    header("HTTP/1.1 400 BAD REQUEST", true, 400); // no document specified - can't check for annotability
    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: Dokumenttia ei ole valittu').'</errormessage>';
}

// Check for annotability being off. In such case, no operations should happen
else if ($rights == ANNO_RIGHTS_NORMAL && !anno_get_annotation_status($document_id)){
    header("HTTP/1.1 400 BAD REQUEST", true, 400);  //annotation off and no rights to turn it on
    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Annoitointia ei ole kytketty plle tlle dokumentille.').'</errormessage>';
}

// Basic things are OK, find out what we are supposed to be doing
else {
    /** Adding a new comment */
    if ($action == 'new_comment') {
        // Read the rest of the parameters in
        $text  = stripslashes(optional_param('text',  "", PARAM_RAW)); // Clean this parameter to prevent script injection
        $place = stripslashes(optional_param('place', "", PARAM_RAW)); // Strip slashes to avoid brokenness
        $category = optional_param('category', "", PARAM_RAW);

        // Verify parameter validity (note text can be empty)
        if ($place == '' || $document_id == '' || $category == '') {
            header("HTTP/1.1 400 BAD REQUEST", true, 400);
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Uutta kommenttia ei voida list koska parametrej puuttuu.').'</errormessage>';
        } else {
            // Try to add send the new comment to Annotea server and pass on the results
            $comment_id = anno_add_comment($document_id, $text, $user, $place, $category);
            if($comment_id == false) {
                header("HTTP/1.1 500 Internal Server Error", true, 500);
                echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: kommenttia ei voitu list.').'</errormessage>';
            } else {
                header("HTTP/1.1 200 OK", true, 200);
                echo '<?xml version="1.0" encoding="UTF-8"?><commentid>'. $comment_id . '</commentid>'; // return it for test scripts
            }
        }
    }


    /** Adding a new message */
    if ($action == 'reply') {
        // Read the parameters
        $text      = stripslashes(optional_param('text', NULL, PARAM_RAW));  // Clean to avoid possible attacks
        $mother_id = optional_param('mother_id', NULL, PARAM_RAW);
        $root_id   = optional_param('root_id',   NULL, PARAM_RAW);

        // Parameter validity check
        if ($text == '' || $mother_id == '' || $root_id == '') {
            header("HTTP/1.1 400 BAD REQUEST", true, 400);
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Uutta kommenttia ei voida list koska parametrej puuttuu.').'</errormessage>';
        } else {
            // Try to add the new reply to Annotea server and pass the results on to the user
            $message_id = anno_add_message($root_id, $text, $user, $mother_id);
            if ($message_id==false){
                header("HTTP/1.1 500 Internal Server Error", true, 500);
                echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: kommenttia ei voitu list.').'</errormessage>';
            } else {
                header("HTTP/1.1 200 OK", true, 200);
                echo '<?xml version="1.0" encoding="UTF-8"?><messageid>'. $message_id . '</messageid>'; // return it for test scripts
            }
        }
    }


    /** Modifying an existing message or comment */
    if ($action == 'modify') {
        // Read parameters
        $type       = optional_param('type',       NULL, PARAM_RAW);
        $id         = optional_param('id',         NULL, PARAM_RAW);
        $text       = stripslashes(optional_param('text', NULL, PARAM_RAW)); // Clean to avoid possible attacks
        $place      = optional_param('place',      NULL, PARAM_RAW);

        if ($place != NULL) {
            $place = stripslashes($place);
        }

        // Parameter validity
        if ($text == '' || $type == '' || $id == '' || ($type != 'message' && $type != 'comment')) {
            header("HTTP/1.1 400 BAD REQUEST", true, 400);
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kommenttia ei voida muokata koska parametrej puuttuu.').'<errormessage>';
        } else {
            // Verify that user can modify the targeted message
            if ($rights != ANNO_RIGHTS_ALL && (!anno_check_ownership($id))){
                header("HTTP/1.1 401 Unauthorized", true, 401);
                echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kyttjll ei ole oikeutta muokata tt kommenttia.').'</errormessage>';
            } else {
                if ($type=='message') { // modifying a message
                    if (anno_modify_message($id, $text, $user)) {
                        header("HTTP/1.1 200 OK", true, 200);  // succesfully modified
                    } else {
                        header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
                        echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: kommentin muokkaus eponnistui.').'</errormessage>';
                    }
                } else if ($type=='comment') { //modifying a comment
                    if(anno_modify_comment($id, $text, $user, $document_id, $place)) {
                        header("HTTP/1.1 200 OK", true, 200); //succesfully modified
                    } else {
                        header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
                        echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: kommentin muokkaus eponnistui.').'</errormessage>';
                    }
                }
            }
        }
    }


    /** Removing an existing message or comment */
    if($action == 'delete') {
        $id = optional_param('id', NULL, PARAM_RAW);

        // Parameter validity check
        if ($id=='') {
            header("HTTP/1.1 400 Bad Request", true, 400); //broken parameters
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kommenttia ei voida poistaa koska parametrej puuttuu.').'</errormessage>';
        } else {
            // this if .. else if check must be done in this order
            // otherwise admin won't be able to remove her/his own comments recursively
            if($rights == ANNO_RIGHTS_ALL) {
                //admin can remove any comment
                $del_annotea = anno_delete_rec($id); //delete recursively from annotea

                // delete from anno_db
                /*  @todo the return value is useless, because:
                    1. Messages are never marked to the database
                    2. Trying to remove something non-existing from the database gives the same
                       return value as a failed query
                    3. We have no way to determine wheather this is a comment or a message, because
                       the ui_module doesn't tell it us.
                       => return value may be false even if everything is fine

                    What is done:
                    - Discard the return value:

                    Side effects:
                    - Failing to remove data from the comment history may in some rare situation
                      lead to marking the unread comment as read.The document, user and even the
                      comment identifier (which should be unique anyway) should be the same for such
                      a thing to occur

                    Howto fix properly:
                    ui_module should make a difference between a comment and a message:
                    1. make anno_handle_request.php to print different event handlers for
                       deleting a comment and a message
                    2. fix event handlers in anno_event_handlers.js
                    3. fix anno_delete_comment in anno_lib.js
                    4. fix this function
                */
                anno_remove($id);

                if($del_annotea){
                    header("HTTP/1.1 200 OK", true, 200);
                } else {
                    header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
                    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: poistaminen eponnistui.').'</errormessage>';
                }
            } else if (!anno_is_comment_locked($id) && !anno_has_replies($id)) { //owner of comment can remove it
		
		$del_annotea = anno_delete($id); //delete from annotea

                 // delete from anno_db
                /*  @todo the return value is useless, because:
                    1. Messages are never marked to the database
                    2. Trying to remove something non-existing from the database gives the same
                       return value as a failed query
                    3. We have no way to determine wheather this is a comment or a message, because
                       the ui_module doesn't tell it us.
                       => return value may be false even if everything is fine

                    What is done:
                    - Discard the return value:

                    Side effects:
                    - Failing to remove data from the comment history may in some rare situation
                      lead to marking the unread comment as read. The document, user and even the
                      comment identifier (which should be unique anyway) be the same for such a thing to occur

                    Howto fix properly:
                    ui_module should make a difference between a comment and a message:
                    1. make anno_handle_request.php to print different event handlers for
                       deleting a comment and a message
                    2. fix event handlers in anno_event_handlers.js
                    3. fix anno_delete_comment in anno_lib.js
                    4. fix this function
                */

		// remove comment from the sql db
		anno_remove($id);

                if ($del_annotea) {
                    header("HTTP/1.1 200 OK", true, 200); //successfully removed
                } else {
                    header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
		    // @todo Proper message to user.
		    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: poistaminen eponnistui.').'</errormessage>';
                }
            }
        }
    }


    /** Switching annotating on/off. R1 and R2 from radiobuttons */
    if ($action == 'set_annotation_on' || $action == 'set_annotation_off') {
        // Check that the user is actually allowed to do this
        if ($rights != ANNO_RIGHTS_ALL && $rights != ANNO_RIGHTS_ALTER_ANNOTABILITY) {
            header("HTTP/1.1 401 Unauthorized", true, 401);
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kyttjll ei ole oikeuksia kytke annotointia plle/pois.').'</errormessage>';
        } else {
            // User does have the permission, see which way the toggle should be turned
            if ($action == 'set_annotation_on') {
                if (anno_set_annotation_status(true, $document_id)) {
                    header("HTTP/1.1 200 OK", true, 200); //succeeded
                } else {
                    header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
                    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: annotoinnin kytkeminen eponnistui.').'</errormessage>';
                }
            } else if ($action == 'set_annotation_off') { //switching off
                if (anno_set_annotation_status(false, $document_id)) {
                    header("HTTP/1.1 200 OK", true, 200);
                } else {
                    header("HTTP/1.1 500 Internal Server Error", true, 500); //did not succeed
                    echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: annotoinnin kytkeminen eponnistui.').'</errormessage>';
                }
            }
        }
    }


    /** Marking comments and messages as read, when closing modifying- or commenting view */
    if ($action == 'mark_read'){
        // Read parameters
        $comment_id = optional_param('id', NULL, PARAM_RAW);

        // Verify parameter validity. Document ID was already checked at the beginning of the script
        if($comment_id == '') {
            header("HTTP/1.1 400 BAD REQUEST", true, 400);  //broken parameters
            echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Kommenttia ei voitu merkit luetuksi koska parametrej puuttuu.').'</errormessage>';
        } else {
            // Parse username
            $user_id_n_name = anno_parse_user_id($user);

            // Do the DB operation
            if(anno_mark_read($user_id_n_name['id'], $comment_id, $document_id)) {
                header("HTTP/1.1 200 OK", true, 200); //successfully marked
                anno_do_response('');
            } else {
                header("HTTP/1.1 500 Internal Server Error", true, 500);
                echo '<?xml version="1.0" encoding="UTF-8"?><errormessage>'.utf8_encode('Sisinen virhe: kommentin luetuksimerkitseminen eponnistui').'</errormessage>';
            }
        }
    }
}

?>
