<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Functions that add Anno-data to a requested document before Moodle sends 
 * it to client.
 * 
 * @author  Heikki Aitakangas, Mika Holmstrm, Kirsi Jokisalo, Timo Myyrylinen, Miro Wikgren, Jukka Huhta
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */


// This is a reference to the DomDocument created in anno_modify_page, used in anno_strip_whitespace_nodes
$domtree="";


/// Functions


/**
 * Converts UTF8 string to html compatible string
 *
 * @param  string $str UTF8 string
 * @return string html compatible string
 */
function utf2html ($str) {
    $ret = "";
    $max = strlen($str);
    $last = 0;  // keeps the index of the last regular character
    for ($i=0; $i<$max; $i++) {
        $c = $str{$i};
        $c1 = ord($c);
        if ($c1>>5 == 6) {  // 110x xxxx, 110 prefix for 2 bytes unicode
            $ret .= substr($str, $last, $i-$last); // append all the regular characters we've passed
            $c1 &= 31; // remove the 3 bit two bytes prefix
            $c2 = ord($str{++$i}); // the next byte
            $c2 &= 63;  // remove the 2 bit trailing byte prefix
            $c2 |= (($c1 & 3) << 6); // last 2 bits of c1 become first 2 of c2
            $c1 >>= 2; // c1 shifts 2 to the right
            $ret .= "&#" . ($c1 * 0x100 + $c2) . ";"; // this is the fastest string concatenation
            $last = $i+1;     
        } else if ($c1>>4 == 14) {  // 1110 xxxx, 110 prefix for 3 bytes unicode
            $ret .= substr($str, $last, $i-$last); // append all the regular characters we've passed
            $c2 = ord($str{++$i}); // the next byte
            $c3 = ord($str{++$i}); // the third byte
            $c1 &= 15; // remove the 4 bit three bytes prefix
            $c2 &= 63;  // remove the 2 bit trailing byte prefix
            $c3 &= 63;  // remove the 2 bit trailing byte prefix
            $c3 |= (($c2 & 3) << 6); // last 2 bits of c2 become first 2 of c3
            $c2 >>=2; //c2 shifts 2 to the right
            $c2 |= (($c1 & 15) << 4); // last 4 bits of c1 become first 4 of c2
            $c1 >>= 4; // c1 shifts 4 to the right
            $ret .= '&#' . (($c1 * 0x10000) + ($c2 * 0x100) + $c3) . ';'; // this is the fastest string concatenation
            $last = $i+1;     
        }
    }
    return $ret . substr($str, $last, $i); // append the last batch of regular characters
} 


/**
 * This adds all the Anno functionalies to the parsed html file it gets from Moodle.
 * This is called every time a page is loaded from Moodle.
 *
 * @param  string $htmlpage   html-page from Moodle
 * @param  string $documentid document's id from Moodle
 * @param  string $userid     user's id from Moodle
 * @return string modified html back to Moodle
 */
function anno_modify_page($htmlpage, $documentid, $userid) {
    global $domtree;
    // Read user rights from Moodle
    $rights = anno_get_user_rights();

    // Retrieve annotability state
    $status = anno_get_annotation_status($documentid); //boolean

    // Annotatibility is off and user can't turn it on -> do nothing
    if (!$status && ($rights == ANNO_RIGHTS_NORMAL || $rights == ANNO_RIGHTS_VISITOR)) {
        return $htmlpage;
    }

    // Create header rows.
    anno_create_header_rows($htmlpage);

    // Create a anno banner and add it to the top of the page.
    anno_create_top_banner($htmlpage, $rights, $status);

    // Create indicator element
    anno_create_indicator($htmlpage);

    // If annotation is off, leave adding controls and other stuff at the banner. Otherwise, proceed here
    if ($status) {
        // Create javascript rows.
        anno_add_javascript($htmlpage);

        // Adds onload event to the body tag
        anno_add_init($htmlpage);

        // Create a comment type selector to the end of the page.
        anno_create_comment_type_selector($htmlpage);

        // Read the comments made to this document from the Annotea server
        $comments = anno_get_comments($documentid);

        // Adds information on comments in a javascript array
        anno_add_comment_array($htmlpage, $comments);

        // Create and add a comment view to the end of the page.
        $commentview = anno_create_comment_views($comments, $rights, $userid, $documentid);

        $end       = utf8_strpos($htmlpage, '</body>');
        $htmlpage  = utf8_substr($htmlpage, 0, $end) . $commentview . utf8_substr($htmlpage, $end);

        // Create and add a view for creating new comments.
        anno_add_new_comment_view($htmlpage);

        // Create and add a view for big window ("suurennettu").
        anno_add_big_view($htmlpage);

        // Create and add view for replying.
        anno_add_reply_view($htmlpage);

        // Create multiview pane 
        // Also create messagebox for errormessages.
        anno_create_multiview_pane($htmlpage);

        // Create a DOM tree of this document.
        $domtree = new DOMDocument();

        @$domtree->loadHTML($htmlpage); // Moodle may generate some not-quite-valid html, so prevent this from whining about it

	// for stripping unnecessary whitespace text nodes from the content
	anno_strip_whitespace($domtree);

        // Add comments, inside <span> elements, to the document.
        // Adds also javascript listeners for mouse.
        anno_place($domtree, $comments, anno_comment_history($userid, $documentid));

        // When all of the comments are added into the DOM tree,
        // convert the tree back to HTML.
        $htmlpage = $domtree->saveHTML();
    }
    return $htmlpage;
}

/**
 * Gets the content div node from the dom tree and feeds it to 
 * the recursive white space remover, anno_strip_whitespace_nodes.
 *
 * @param &$doc, a pointer to the dom tree
 */
function anno_strip_whitespace(&$doc) {
    $content =& $doc->getElementById('content');
    if ($content->tagName != "div") {
        return;
    }
    anno_strip_whitespace_nodes($content);
}

/**
 * Parses recursively all given nodes and their child nodes and
 * removes any whitespace from text nodes.
 *
 * @param &$node a pointer to a node to parse
 */
function anno_strip_whitespace_nodes(&$node) {
    global $domtree;

    if ($node == null) { return; }

    if ($node->nodeName == "br" && ($node->nextSibling->nodeType != XML_TEXT_NODE)) { 

        // Insert a text node with two spaces after the current <br> node
        // So for example "<br><i>foo</i>" becomes "<br>  <i>foo</i>"

        $tmpNode=$domtree->createTextNode("  ");
        if ($node->nextSibling==null) { // This is last child, so insert the text node to the end
            $node->parentNode->appendChild($tmpNode);
        } else {                        // not last child, so insert text node before next node 
            $node->parentNode->insertBefore($tmpNode,$node->nextSibling);
        }
    } else if ($node->nodeType == XML_TEXT_NODE && ($node->parentNode->nodeName != "pre"
                                                && $node->parentNode->nodeName != "textarea"
                                                && $node->parentNode->nodeName != "optgroup"
                                                && $node->parentNode->nodeName != "select")) { 

        $node->nodeValue = preg_replace("/\s+/", " ", $node->nodeValue); // remove duplicate whitespaces

        if ($node->previousSibling==null) { // This is first child
            $node->nodeValue = preg_replace("/^\s+/", "", $node->nodeValue); // remove all whitespace after start tag
        }
        if ($node->nextSibling==null) { // This is last child
            $node->nodeValue = preg_replace("/\s+$/", "", $node->nodeValue); // remove all whitespace before end tag
        }
	if ($node->previousSibling->nodeName == "br") { // make sure this are two spaces after <br> tag
            $node->nodeValue = preg_replace("/^\s*/", "  ", $node->nodeValue);
        }
    }

    foreach ($node->childNodes as $child) {
        anno_strip_whitespace_nodes($child);
    }
}


/**
 * This function checks the ownership between a user and a comment/message
 *
 * @param  string  $id   creator id
 * @param  string  $user user id
 * @return boolean true if given user is the creator.
 */
function anno_is_owner($id, $user) {
    $message = anno_get_single_message_base_data($id);
    return $user == $message->creator;
}


/**
 * This function parses timestamp to a more clear form
 *
 * @param  string $stamp timestamp
 * @return string parsed timestamp 
 */
function anno_parse_timestamp($stamp) {
    return strftime("%Y-%m-%d, %H:%M:%S", strtotime($stamp));
}


/**
 * Creates a comment view for each comment in the document.
 * 
 * @param array  $comment kommenttipuu
 * @param string $rights  user's rights
 * @param string $userid  user id
 * @param string $documentid document's id
 * @return string commentviews in html
 */
function anno_create_comment_views($comments, $rights, $userid, $documentid) {
    // All of the views will be saved in this variable.
    $commentviews = '';

    // Each comment is handled separately in a function which returns
    // a finished comment view of a given comment. This loop just
    // puts those different comment views together.
    foreach ($comments as $comment) {
        $commentviews .= anno_create_comment_view($comment, $rights, $userid, $documentid);
    }
    return $commentviews;
}


/**
 * Goes through a given comment and creates html elements according to
 * a specific syntax.
 *
 * The first message in each comment is special and, therefore, will be
 * handled differently than other messages (if there are more).
 *
 * The parameter $comment is of type AnnoMessageData and has the following attributes:
 *       $id;           A Unique id.
 *       $creator;      Name of the creator of this message.
 *       $date;         Whn last modified in W3C date format.
 *       $content;      Content of the message.
 *       $position;     XPointer location if this is a comment.
 *       $category;     Category of the comment, if this is a comment.
 *       $replies       AnnoMessageData objects representing replies to this message.
 * 
 * @param array  $comment kommenttipuu
 * @param string $rights  user's rights
 * @param string $userid  user id
 * @param string $documentid document's id
 * @return string commentviews in html
 */
function anno_create_comment_view($comment, $rights, $userid, $documentid) {

    // we need to check if the comment is read or not 
    $readbyuser=anno_comment_history($userid, $documentid); 
    $lastreaddate=$readbyuser[$comment->id];

    // Determine the latest date the comment tree starting at the current comment has been modified
    $lastmodified = anno_get_last_modified($comment);
     
    // See if it should be marked as read 
    if (strtotime($lastreaddate) > $lastmodified) {
        $commentisread = true; 
    } else { 
        $commentisread = false; 
    } 

    // Parse name out of the author's user id
    $authorparse = anno_parse_user_id($comment->creator);

    // The view will be saved here
    $commentview = '';

    // First we need to create a div element, where all of the messages
    // will be put into, and give it a right commentId.

    $commentview .= '<div class="anno_popup" id="anno_popup_' . $comment->id . '">' . "\n";
    $commentview .= '  <div class="anno_frame" id="anno_frame_' . $comment->id . '">' . "\n";
    $commentview .= '  <div class="anno_frame_fit">' . "\n";

    // The first message has no bar on its left side
    
    $commentview .= '    <div class="anno_text_first">' . "\n";
    $commentview .= '      <table class="anno_message_content" id="anno_comment_view_' . $comment->id . '">' . "\n";
    $commentview .= '        <tr>' . "\n";
    $commentview .= '          <td id="anno_message_author_created_' . $comment->id . '" class="anno_message_author_created">' . "\n";
    $commentview .= '            <div class="anno_message_author">' . $authorparse['name'] . '</div>' . "\n";
    $commentview .= '            <div class="anno_message_created">' . anno_parse_timestamp($comment->date) . '</div>' . "\n";
    $commentview .= '          </td>' . "\n";

    // Add reply and remove links according to user rights
    if ($rights == ANNO_RIGHTS_NORMAL || $rights == ANNO_RIGHTS_ALTER_ANNOTABILITY || $rights == ANNO_RIGHTS_ALL) {

        // Logged users are allowed to create replies
        $commentview .= '          <td class="anno_message_action"><input type=button onclick="anno_on_click_respond(\'' 
	    . $comment->id . '\', \'' . $comment->id . '\');" value="Vastaa">&nbsp;';

        // Only the owner of a message or comment can remove it and only if the message has no replies
	// and if the message was made less than ANNO_COMMENT_REMOVAL_TIME minutes ago.
        // User with "ALL RIGHTS" may remove any comment and messages he/she wants to, no matter are they last or not.
        if (($userid == $comment->creator 
	     && $comment->replies == null 
	     && !anno_is_comment_locked($comment->id)) 
	    || $rights == ANNO_RIGHTS_ALL) {
            $commentview .= '<input type=button onclick="anno_on_click_delete(event, \'' 
		. $comment->id . '\');" value="Poista">';
        }
    }
    $commentview .= '        <td class="anno_message_spacer">&nbsp;</tr>' . "\n";

    // this is just storage for the style information
    if ($commentisread) {
        $commentview .= '        <tr><td id="anno_comment_style_' . $comment->id . '" class="anno_multiview_' . $comment->category . '_read"></td>' . "\n";
    } else {
        $commentview .= '        <tr><td id="anno_comment_style_' . $comment->id . '" class="anno_multiview_' . $comment->category . '_unread"></td>' . "\n";
    }
    $commentview .= '        <tr>' . "\n";

    // added the str_replace to circumvent what seems to be a bug in PHP.
    $commentview .= '          <td colspan="3" class="anno_message_text" id="anno_message_text_' . $comment->id . '" >' . 
                    str_replace('&amp;apos;','&apos;', utf2html(htmlspecialchars($comment->content,ENT_NOQUOTES))) . '</td>' . "\n";
    $commentview .= '        </tr>' . "\n";
    $commentview .= '      </table>' . "\n";
    $commentview .= '    </div>' . "\n";

    // If this comment has more than one message, which would mean there are
    // replies or a reply to the original comment, then their place is here.

    if ($comment->replies != null) {
      
        // This variable will be given as a parameter to tell are we working on the first child or not.
        $firstchild = true;

        // This needs to be done in reverse order, because annotea gives the newest msgs first.

        $replies = array_reverse($comment->replies);
        foreach ($replies as $message) {
            $commentview .= anno_create_comment_messages($comment, $message, $rights, $userid, 1, $firstchild);
            $firstchild = false;
        }
    }

    // When all the replies and replies' replies (and so on) are included,
    // we can close the div element, because now we've gone through all the
    // messages that belong to this specific comment.
    $commentview .= '  </div>' . "\n";
    $commentview .= '  </div>' . "\n";

    // At the end of each comment view, we have to add a footer for the "enlarge" button.
    $commentview .= '  <div class="anno_popup_footer">' . "\n";
    $commentview .= '    <input type=button onclick="anno_on_click_show_big_view(\'' . $comment->id . '\');" value="Suurenna">' . "\n";
    $commentview .= ' <input type=button onclick="anno_on_mouse_over_comment_multiple(event, anno_last_multiview_id);" value="Palaa" 
class="anno_return_to_multiple" id="anno_return_to_multiple_' . $comment->id . '">';
    $commentview .= '  </div>' . "\n";

    // Now we've finished building a comment view, so let's
    // close the final div element and return the view.
    $commentview .= '</div>' . "\n";

    return $commentview;
}


/**
 * Returns the view of a given message and its replies,
 * replies's replies and so on.
 *
 * @param object $comment comment
 * @param object $message message
 * @param string $rights  user's rights
 * @param string $userid  user id
 * @param string $deph    message
 * @param boolean $firschild is this first child message
 * @return string messageview in html
 */
function anno_create_comment_messages($comment, $message, $rights, $userid, $depth, $firstchild) {

    // Parse name out of the author's user id
    $authorparse = anno_parse_user_id($message->creator);

    // The view will be saved here
    $messageview = '';

    // There are two possibilities what to do next:
    // 1. if this message (reply) is the first child of some other message,
    //    then we'll need to create a new vertical bar to show this visually.
    // 2. if this message is not the first child, we'll just add it's content here
    if ($depth%2 == 0) {
        $messageview .= '    <div class="anno_bar_odd">' . "\n";
    } else {
        $messageview .= '    <div class="anno_bar_even">' . "\n";
    }

    if ($firstchild) {
        $messageview .= '      <div class="anno_text_first">' . "\n";
        $messageview .= '      <table class="anno_message_content" id="anno_comment_view_' . $message->id . '">' . "\n";
        $messageview .= '        <tr>' . "\n";
        $messageview .= '          <td class="anno_message_author_created" style="width: '.(12-$depth*0.6).'em">' . "\n";
        $messageview .= '            <div class="anno_message_author">' . $authorparse['name'] . '</div>' . "\n";
        $messageview .= '            <div class="anno_message_created">' . anno_parse_timestamp($message->date) . '</div>' . "\n";
        $messageview .= '          </td>' . "\n";

        // Add reply and remove  links according to user rights.
        if ($rights == ANNO_RIGHTS_NORMAL || $rights == ANNO_RIGHTS_ALTER_ANNOTABILITY || $rights == ANNO_RIGHTS_ALL) {

            // Logged users are allowed to create replies
            $messageview .= '          <td class="anno_message_action"><input type=button onclick="anno_on_click_respond(\'' . $comment->id . '\', \'' 
. $message->id . '\');" value="Vastaa">&nbsp;';

            // Only the owner of a message or comment can remove it and only if the message has no replies
	    // and if the message was made less than ANNO_COMMENT_REMOVAL_TIME minutes ago.
            // User with "ALL RIGHTS" may remove any comment and messages he/she wants to, no matter are they last or not.
            if ((($userid == $message->creator) 
                 && (count($message->replies)==0)
		 && (!anno_is_comment_locked($message->id))) 
		|| ($rights == ANNO_RIGHTS_ALL)) {
               $messageview .= '<input type=button onclick="anno_on_click_delete(event, \'' 
		   . $message->id . '\');" value="Poista">';
            }
        }

        $messageview .= '        <td class="anno_message_spacer">&nbsp;</tr>' . "\n";
        $messageview .= '        <tr>' . "\n";
        $messageview .= '          <td class="anno_message_text" colspan="3">' . $message->content . '</td>' . "\n";
        $messageview .= '        </tr>' . "\n";
        $messageview .= '      </table>' . "\n";
        $messageview .= '      </div>' . "\n";

        // If this message has replies they will be handled next
        if ($message->replies != null) {
            // This variable will be given as a parameter to tell are we working on the first child or not.
            $firstchild = true;

            $replies = array_reverse($message->replies);

            foreach ($replies as $reply) {
                $messageview .= anno_create_comment_messages($comment, $reply, $rights, $userid, $depth+1, $firstchild);
                $firstchild = false;
            }
        }

        // No more replies, let's close this element and return it.
        $messageview .= '    </div>' . "\n";
    } else {
        $messageview .= '      <div class="anno_text">' . "\n";
        $messageview .= '      <table class="anno_message_content" id="anno_comment_view_' . $message->id . '">' . "\n";
        $messageview .= '        <tr>' . "\n";
        $messageview .= '          <td class="anno_message_author_created" style="width: '.(12-$depth*0.6).'em">' . "\n";
        $messageview .= '            <div class="anno_message_author">' . $authorparse['name'] . '</div>' . "\n";
        $messageview .= '            <div class="anno_message_created">' . anno_parse_timestamp($message->date) . '</div>' . "\n";
        $messageview .= '          </td>' . "\n";

        // Add reply and remove  links according to user rights.
        if ($rights == ANNO_RIGHTS_NORMAL || $rights == ANNO_RIGHTS_ALTER_ANNOTABILITY || $rights == ANNO_RIGHTS_ALL) {

            // Logged users are allowed to create replies
            $messageview .= '          <td class="anno_message_action"><input type=button onclick="anno_on_click_respond(\'' . $comment->id . '\', \'' 
. $message->id . '\');" value="Vastaa">&nbsp;';

            // Only the owner of a message or comment can remove it and only if the message has no replies
	    // and if the message was made less than ANNO_COMMENT_REMOVAL_TIME minutes ago.
            // User with "ALL RIGHTS" may remove any comment and messages he/she wants to, no matter are they last or not.
            if ((($userid == $message->creator) 
                 && (count($message->replies)==0)
		 && (!anno_is_comment_locked($message->id))) 
		|| $rights == ANNO_RIGHTS_ALL) {
                $messageview .= '<input type=button onclick="anno_on_click_delete(event, \'' 
		   . $message->id 
		   . '\');" value="Poista">';
            }
        }

        $messageview .= '        <td class="anno_message_spacer">&nbsp;</tr>' . "\n";
        $messageview .= '        <tr>' . "\n";
        $messageview .= '          <td class="anno_message_text" colspan="3">' . $message->content . '</td>' . "\n";
        $messageview .= '        </tr>' . "\n";
        $messageview .= '      </table>' . "\n";
        $messageview .= '      </div>' . "\n";
        
        // If this message has replies they will be handled next
        if ($message->replies != null) {
            // This variable will be given as a parameter to tell are we working on the first child or not.
            $firstchild = true;

            $replies = array_reverse($message->replies);

            foreach ($replies as $reply) {
                $messageview .= anno_create_comment_messages($comment, $reply, $rights, $userid, ($depth+1), $firstchild);
                $firstchild = false;
            }
        }
        $messageview .= '    </div>' . "\n";
    }
    return $messageview;
}


/**
 * Prints the Anno banner at the top of a page that's being modified
 * 
 * @param string &$htmlpage pointer to the html page
 * @param string $rights    user's rights
 * @status boolean $status  is annotation on/off
 */
function anno_create_top_banner(&$htmlpage, $rights, $status) {
    $banner = <<<EOF
  <table class="anno_toolbar">
    <tr>
      <td>
        <span class="anno_toolbar_shorttext">Maalaa alue, jota haluat kommentoida</span>
EOF;

    // If the user has a right to turn annotation on/off
    // the banner will have an option for this
    if ($rights == ANNO_RIGHTS_ALTER_ANNOTABILITY || $rights == ANNO_RIGHTS_ALL) {
        if ($status) {
            $banner .=    '<span class="anno_toolbar_on_off">' .
         				  '  <input name="annotable" type="radio" value="on" onclick="anno_on_click_toggle_annotation(true);"  checked="checked" /> Kommentointi k&auml;yt&ouml;ss&auml; ' .
        				  '  <input name="annotable" type="radio" value="off" onclick="anno_on_click_toggle_annotation(false);" />Kommentointi ei k&auml;yt&ouml;ss&auml; ' .
        				  '</span>' . "\n";
	} else {
	    $banner .=    '<span class="anno_toolbar_on_off">' .
         				  '  <input name="annotable" type="radio" value="on" onclick="anno_on_click_toggle_annotation(true);" /> Kommentointi k&auml;yt&ouml;ss&auml; ' .
        				  '  <input name="annotable" type="radio" value="off" onclick="anno_on_click_toggle_annotation(false);" checked="checked" />Kommentointi ei k&auml;yt&ouml;ss&auml; ' .
        				  '</span>' . "\n";
        }
    }

    $banner .= <<<EOF
      </td>
      <td><span class="anno_toolbar_help"><a href="#" onclick="anno_on_click_help();">Ohje</a></span></td>
    </tr>
  </table>
EOF;

    // The banner will be added just before the div element which id is "content".
    $pos = utf8_strpos($htmlpage, '<div id="content"');
    $htmlpage = utf8_substr($htmlpage, 0, $pos) . $banner . utf8_substr($htmlpage, $pos);
}


/**
 * Adds a 'new comment view' to the page
 * 
 * @param string &$htmlpage pointer to the html page
 */
function anno_add_new_comment_view(&$htmlpage) {
    $view = <<<EOF
<div class="anno_view_pane" id="anno_view_pane_new_comment">
  <table class="anno_view_pane">
    <tr class="anno_view_pane_top">
      <td><span class="anno_close_viewpane"><input type=button onclick="anno_close_view_pane_new_comment()" value="Sulje"></span></td>
    </tr>
    <tr class="anno_view_pane_bottom"> 
      <td><table class="anno_new_comment">
          <tr class="anno_new_comment_top">
            <td id="anno_new_comment_description">Kirjoita uusi kommentti</td>
          </tr>
          <tr class="anno_new_comment_middle">
            <td class="anno_new_comment"><textarea rows=5 id="anno_new_comment_textarea" class="anno_new_comment" name="text"></textarea></td>
          </tr>
          <tr class="anno_new_comment_bottom">
            <td><div class="anno_center"><input id="anno_view_pane_button" onclick="anno_on_click_publish();" type="submit" name="Submit" value="Julkaise"></div></td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
</div>
EOF;

    // Add this element before the end of body tag.
    $end       = utf8_strpos($htmlpage, '</body>');
    $htmlpage  = utf8_substr($htmlpage, 0, $end) . $view . utf8_substr($htmlpage, $end);
}


/**
 * Creates a view for "suurennettu" (big) 
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_add_big_view(&$htmlpage) {
    $view = <<<EOF
<div class="anno_view_pane" id="anno_view_pane_big">
  <table class="anno_view_pane">
    <tr class="anno_view_pane_top">
      <td><span class="anno_close_viewpane"><input type=button onclick="anno_close_view_pane_big();" value="Sulje"></span></td>
    </tr>
    <tr class="anno_view_pane_bottom"> 
      <td id="anno_view_pane_big_comment_tree">
	  
	  </td>
    </tr>
  </table>
</div>
EOF;

    // Add this element before the end of body tag.
    $end       = utf8_strpos($htmlpage, '</body>');
    $htmlpage  = utf8_substr($htmlpage, 0, $end) . $view . utf8_substr($htmlpage, $end);	
}


/**
 * Creates a view for replying.
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_add_reply_view(&$htmlpage) {
    $view = <<<EOF
<div class="anno_view_pane" id="anno_view_pane_reply">
  <table class="anno_view_pane">
    <tr class="anno_view_pane_top">
      <td><span class="anno_close_viewpane"><input type=button onclick="anno_close_view_pane_reply()" value="Sulje"></span></td>
    </tr>
    <tr class="anno_view_pane_bottom"> 
      <td><table class="anno_comment_view_divided">
          <tr> 
            <td class="anno_comment_view_left" id="anno_view_pane_reply_tree"> 	
		</td>
            <td class="anno_comment_view_right"> 
			
			    <table class="anno_create_reply">
                <tr class="anno_new_comment_top"> 
                  <td>Kirjoita vastaus:</td>
                </tr>
                <tr class="anno_new_comment_middle"> 
                  <td class="anno_new_comment"><textarea rows=5 id="anno_new_reply_textarea" class="anno_new_comment" name="textarea"></textarea></td>
                </tr>
                <tr class="anno_new_comment_bottom"> 
                  <td><div class="anno_center"> 
                      <input id="Submit" onclick="anno_on_click_publish();" type="submit" name="Submit2" value="Julkaise" />
                    </div></td>
                </tr>
              </table></td>
          </tr>
        </table></td>
    </tr>
  </table>
</div>
EOF;

    // Add this element before the end of body tag.
    $end       = utf8_strpos($htmlpage, '</body>');
    $htmlpage  = utf8_substr($htmlpage, 0, $end) . $view . utf8_substr($htmlpage, $end);
}


/**
 * Adds javascript functionality to the content element
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_add_javascript(&$htmlpage) {
    $javascript = ' onMouseDown="anno_on_mouse_down_document(event)" onMouseUp="anno_on_mouse_up_document(event)"';

    // Find the right place in content element and put our script thing in there.
    $pos = utf8_strpos($htmlpage, '<div id="content"');
    $htmlpage = utf8_substr($htmlpage, 0, $pos+17) . $javascript .utf8_substr($htmlpage, $pos+17) ;
}


/**
 * Adds a javascript functionality to the body element.
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_add_init(&$htmlpage) {
    $init = ' onload="anno_init();"';

    $pos = utf8_strpos($htmlpage, "<body");

    // Anno3 requirement 4.15., the comment type window is closed if the user looses the selected text.
    $htmlpage = utf8_substr($htmlpage, 0, $pos+5) . " onclick='  
  var txt = \"\";
  if (window.getSelection) { // Mozilla/Firefox, Safari
    txt = window.getSelection(); 
  } else if (document.getSelection) { // Explorer 5+ Mac,
                                      // Mozilla/Firefox,
                                      // Opera, Netscape 
    txt = document.getSelection(); 
  } else if (document.selection) { // Explorer 5+ Win
    txt = document.selection.createRange().text; 
  } 
  if (txt==\"\") { document.getElementById(\"anno_comment_type_selector\").style.display=\"none\"; } else { }                                  
                                   
'" . $init . utf8_substr($htmlpage, $pos+5);

}


/**
 * Adds header rows telling where to fetch all required js and css files.
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_create_header_rows(&$htmlpage) {
    global $CFG;

    $rows = <<<EOF
<link rel="stylesheet" type="text/css" href="$CFG->wwwroot/anno/ui_module/anno_styles.css">
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_globals.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_connection.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_lib.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_view_pane.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_comment_view.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_pop_up.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_event_handlers.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_x_pointer_lib.js"></script>
<script language="JavaScript" type="text/javascript" src="$CFG->wwwroot/anno/ui_module/anno_util.js"></script>
<script language="JavaScript">
     anno_server_uri = "$CFG->wwwroot/anno/handler_module/anno_handle_action.php";
     anno_help_uri   = "$CFG->wwwroot/anno/ui_module/anno_help.html";
</script>
EOF;

    // Header rows will be adder just after the <head> tag
    $pos = utf8_strpos($htmlpage, '<head>');
    $htmlpage = utf8_substr($htmlpage, 0, $pos+6) . $rows . utf8_substr($htmlpage, $pos+6);
}


/**
 * Adds information on comments in a javascript array
 *
 * @param string &$htmlpage pointer to the html page
 * @param array  $commentarray array of comments
 */
function anno_add_comment_array(&$htmlpage, $commentarray) {
    // Sanity check
    if ($htmlpage == NULL || $commentarray == NULL) {
        return;
    }
    $rows = "";
    $i = 0;
    foreach ($commentarray as $comment) {
        // Sanity check on the XPointer
        if(!$comment->position || $comment == NULL) {
            continue;
        }
        // Add every comment on this document to a new row
        $x_pointer = addslashes(strstr($comment->position, "id="));
        $rows .= "commentarray[$i] = new Array(2);\n";
        $rows .= "commentarray[$i][0] = \"$x_pointer\";\n";
        $rows .= "commentarray[$i][1] = \"".$comment->category."\";\n";
        $i++;
    }
    // Add the size of the array 
    $rows = "var commentarray = new Array($i);\n".$rows;

    // Add <script> -tags
    $rows = "<!-- // Array of all the comments in this document\n".$rows;
    $rows = "\n<script language=\"JavaScript\" type=\"text/javascript\">\n".$rows;
    $rows .= "// done hiding -->\n";
    $rows .= "</script>\n";

    // Add array just before the </head> -tag
    if ($i > 0) {
        $pos = utf8_strpos($htmlpage, '</head>');
        if ($pos>0) {
            $htmlpage = utf8_substr($htmlpage, 0, $pos) . $rows . utf8_substr($htmlpage, $pos);
        }
    }
}


/**
 * Adds a comment type selector
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_create_comment_type_selector(&$htmlpage) {
    $selector = <<<EOF
<div id="anno_comment_type_selector" class="anno_comment_type_selector"> 
  <table class="moaan_comment_type">
    <tr> 
      <td class="anno_comment_type_text">Valitse kommenttityyppi</td>
      <td><input type=button  onclick="anno_on_click_close_comment_type_selector();" class="anno_comment_type_close" value="X"></td>
    </tr>
    <tr> 
      <td colspan="2"><div class="anno_center"> 
          <input id=anno_sisaltonappi type="button" onClick="anno_on_click_comment_type('content');" class="valinta" 
name="B1" value="Sis&auml;lt&ouml;">
          <input id=anno_oikeinkirjoitus_nappi type="button" onClick="anno_on_click_comment_type('spelling');" 
class="valinta" name="B2" value="Oikeinkirjoitus">
        </div></td>
    </tr>
  </table>
</div>
EOF;

    // Add just before the body tag closes
    $pos = utf8_strpos($htmlpage, '</body>');
    $htmlpage = utf8_substr($htmlpage, 0, $pos) . $selector . utf8_substr($htmlpage, $pos);
}


/**
 * Adds a hidden element that pops up when something is added to the db.
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_create_indicator(&$htmlpage) {
    $ind = <<<EOF
<div class="anno_indicator" id="anno_indicator">
    Ladataan...
</div>
EOF;

    $pos       = utf8_strpos($htmlpage, '</body>');
    $htmlpage  = utf8_substr($htmlpage, 0, $pos) . $ind . utf8_substr($htmlpage, $pos);
}


/**
 * Creates and adds a comment view pane to the htmlpage
 *
 * @param string &$htmlpage pointer to the html page
 */
function anno_create_view_pane(&$htmlpage) {
    $viewpane = <<<EOF
<div class="anno_indicator" id="anno_indicator">
    Ladataan...
</div>

<div class="anno_view_pane" id="anno_view_pane">
  <div class="anno_view_pane_comment_stub" id="anno_view_pane_comment_stub">  </div>

  <div class="anno_view_pane_editor">
    <form id="anno_form">

      <input type="button" class="anno_close" onclick="anno_on_click_close_view_pane()" name="close" value="X" />
      <span id="anno_view_pane_heading" class="teksti"></span>
      <br />
      <textarea class="anno_view_pane_textarea" name="text"></textarea>
      <br />
      <input type="button" id="anno_view_pane_button" onclick="anno_on_click_publish();" name="Button1" class="julkaise" value="Hyv&auml;ksy muokkaus" />
    </form>
  </div>
</div>
EOF;

    $pos       = utf8_strpos($htmlpage, '</body>');
    $htmlpage  = utf8_substr($htmlpage, 0, $pos) . $viewpane . utf8_substr($htmlpage, $pos);
}


/**  
 * Creates and adds the multiview pane ("vli-ikkuna") to the html page
 *
 * @param string &$htmlpage pointer to the html page
 */ 
function anno_create_multiview_pane(&$htmlpage) {
    $viewpane = <<<EOF
<div class="anno_popup" id="anno_popup_multiview">
  <div class="anno_frame" id="anno_frame_multiview">
  <div class="anno_frame_fit">
    <div class="anno_text_first" id="anno_comment_view_multiview_text">
      <table class="anno_message_content" id="anno_comment_view_multiview">
<tr>
<td class="anno_message_author_created">
            <div class="anno_message_author">Author</div>
            <div class="anno_message_created">Date, Time</div>

          </td>
          <td class="anno_message_action"><input type="button" onclick="anno_on_mouse_over_comment(event, 'commentID');" value="Nyt"></td>
        </tr>
<tr>
<td class="anno_message_text" colspan="3">Comment Text</td>
        </tr>
</table>
</div>
  </div>
  </div>
  <div class="anno_popup_footer">
    <input type="button" style="visibility: hidden;" value="">
</div>  
</div>

<div class="anno_errorview" id="anno_errorview">
<table>
<tr><td id="anno_errortext"></td></tr><tr><td><input type=button value="Sulje" 
onclick="document.getElementById('anno_errorview').style.visibility = 'hidden'">
</tr>
</table>
</div> 


EOF;

    $pos       = utf8_strpos($htmlpage, '</body>');
    if ($pos!==FALSE) {
        $htmlpage  = utf8_substr($htmlpage, 0, $pos) . $viewpane . utf8_substr($htmlpage, $pos);
    }
}

?>
