<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Library of functions for interfacing with Moodle.
 *
 * @author Heikki Aitakangas
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * 
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 * 
 */


/// Functions

/**
 * Gets the current user's rights as interpreted from Moodle rights
 * to Anno rights.
 *
 * @return int One of the ANNO_RIGHTS_*
 */
function anno_get_user_rights() {
    // If user is not logged in or logged in as quest
    if (isloggedin() == false || isguest()) {
        return ANNO_RIGHTS_VISITOR;
    } else {
        // Otherwise, figure out what rights they to current document
        // Admins and teachers with editing rights to this course
        if(isadmin() || isteacherinanycourse()) {
            return ANNO_RIGHTS_ALL;
        } else {
            // Normal user
            /** @todo Special handling for wiki (and other modules - if any - where a student can own a page) */
            /*if( *wiki* ) {
            } else */{
                return ANNO_RIGHTS_NORMAL;
            }
        }
    }
}

/**
 * Starts output buffering if the current document is annotable or
 * if it's not annotable but the current user has the rights to turn it on.
 */
function anno_output_buffering_start() {
    global $CFG;

    // Switch error reporting level and start logging
    if (ANNO_SET_ERROR_LEVEL) {
        error_reporting(error_reporting() | E_USER_NOTICE | E_USER_WARNING | E_USER_ERROR);
    }

    // See if we should capture the output
    if (anno_get_annotation_status(anno_get_current_document_id()) || // Document is annotable
        anno_get_user_rights() == ANNO_RIGHTS_ALL                  ||
        anno_get_user_rights() == ANNO_RIGHTS_ALTER_ANNOTABILITY) {
        // Start output buffering
        ob_start();
    }
}

/**
 * Ends output buffering if the same checks as were run in anno_output_buffering_start()
 * before starting it pass.
 * Passes the captured document to anno_modify_page() for handling.
 */
function anno_output_buffering_end() {
    if (anno_get_annotation_status(anno_get_current_document_id()) || // Document is annotable
        anno_get_user_rights() == ANNO_RIGHTS_ALL                  ||
        anno_get_user_rights() == ANNO_RIGHTS_ALTER_ANNOTABILITY) {

        // Read captured output
        $captureddocument = ob_get_clean();

        // Pass the captured data on for further modification
        $captureddocument = anno_modify_page($captureddocument, anno_get_current_document_id(),
                                              anno_get_current_user_id());
        // Print the modified data back out
        echo($captureddocument);
    }
}

/**
 * Returns the request portion of the URI that was used to retrieve this document.
 *
 * @return string  The output of Moodle qualified_me() function
 */
function anno_get_current_document_id() {
    $toreturn = qualified_me();

    // Does the URI contain parameters
    if (strpos($toreturn, '?')) {
        // If so, parse off any added by anno
        $halves = explode('?', $toreturn);
        $pieces = explode('&', $halves[1]);

        $toreturn = $halves[0] . '?';
        $first = true;

        foreach($pieces as $piece) {
            if((substr($piece, 0, 5) != 'anno_') && substr($piece, 0, strlen(ANNO_PREFIX)) != 'ANNO_') { 
                $toreturn .= ($first ? '' : '&') . $piece;
                $first = false;
            }
        }
    }

    return $toreturn;
}

/**
 * Determines the userid of the current user and returns in in
 * "serialised" form:
 * numericalid:fullname
 *
 * @return The current user's ID for use in Anno code, or false if the user has not
 *         logged in
 */
function anno_get_current_user_id() {
    global $USER;

    if(!isloggedin()) {
        return false;
    } else {
        return ((string)$USER->id) . ':' . fullname($USER);
    }
}

/**
 * Parses the numerical id and the user's fullname out of an userid generated
 * by anno_get_current_user_id().
 *
 * @return mixed False on failure, an associative array containing
 *               'id'   => numerical user id
 *               'name' => user full name
 */
function anno_parse_user_id($userid) {
    $splitpos = utf8_strpos($userid, ':');

    // Invalid ID, fail
    if ($splitpos === false) {
        return false;
    }

    $toreturn = array('id'   => utf8_substr($userid, 0, $splitpos),
                      'name' => utf8_substr($userid, $splitpos + 1)
                      );

    // Sanity check
    if (!is_numeric($toreturn['id']) || utf8_strlen($toreturn['name']) == 0) {
        return false;
    }

    // Cast the id from string to integer
    $toreturn['id'] = (int)$toreturn['id'];

    return $toreturn;
}

?>
