<?php                   
/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Library of functions that handle the reading history of the comments.
 *
 * @author  Kirsi Jokisalo, Timo Myyrylinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * 
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 * 
 */


/**
 * This function marks certain comment as read to db. If read the
 * first time, it creates a row for current user&comment. If user
 * has  read the comment before, it updates the readingtime.
 *
 * @param int    $user_id     The user's numeric ID
 * @param string $comment_id  ID of the comment to be marked read by the user
 * @param string $document_id ID of the document where the comment is in
 * @return boolean  True if the sql execution was successfull, false otherwise
 */
function anno_mark_read($user_id, $comment_id, $document_id) {
    // Use the Moodle database connection
    global $db;

    // Check the parameters
    if(!is_string($document_id) || !is_string($comment_id) || $user_id <= 0) {
        trigger_error('anno_mark_read(): Invalid parameters', E_USER_WARNING);
        return false;
    }

    // Lowercase and add slashes to the document ID
    $document_id = addslashes(utf8_strtolower($document_id));

    //get current time. format ISO-8601 date: 2004-02-12T15:19:21+00:00
    $time = date(DATE_W3C, time());

    //query to database
    $comment = $db->Execute('SELECT comment_id FROM anno_comment_history'
                           .' WHERE comment_id=' . "'" . $comment_id . "'"
                           .' AND   user_id='    . "'" . $user_id    . "'"
                            );

    if($comment->fields[0] == $comment_id) {
        // User has read the comment before.
        // Update also document_id to prevent malfunction when annodb holds
        // data that doesn't have relevance to Annotea. This only occurs when
        // Annotea is emptied and annodb is not.
        $sql = 'UPDATE anno_comment_history'
              .' SET   visited='    . "'" . $time       . "'"
              .' , document_id=' ."'". $document_id ."'" 
              .' WHERE comment_id=' . "'" . $comment_id . "'"
              .' AND   user_id='    . "'" . $user_id    . "'";
    } else {
        // reading the first time, adding row to db
        $sql = "INSERT INTO anno_comment_history VALUES ('$comment_id','$document_id','$user_id','$time')";
    }

    $dbresult = $db->Execute($sql);

    if ($dbresult) {
        return true;
    } else {
        return false;
    }
}

/**
 * Reads from the DB which comments attached to the current document the current user
 * has read, and if read, when.
 *
 * @param string $user        The user string, as returned by anno_get_current_user_id()
 * @param string $document_id ID of the document, as returned by anno_get_current_document_id()
 * @return array  An associative array of commentID => date last read. An empty array
 *                is returned on failure.
 */
function anno_comment_history($user, $document_id) {
    // Use the Moodle database connection
    global $db;

    // Get the user number out of the ID
    $userparse = anno_parse_user_id($user);
    $userid = $userparse['id'];

    // Init the return array
    $toreturn = Array();

    // Sanity check on the parameters. Should be unnecessary, since this function should never be called with data
    // provided directly by the user
    if(!is_string($document_id) || $userid <= 0) {
        trigger_error('anno_comment_history(): Invalid parameters', E_USER_WARNING);
        return $toreturn;
    }

    // Lowercase and add slashes to the document ID
    $document_id = addslashes(utf8_strtolower($document_id));

    // query to database
    $dbresult = $db->Execute('SELECT comment_id, visited FROM anno_comment_history'
                            .' WHERE user_id='   . "'" . $userid      . "'"
                            .' AND document_id=' . "'" . $document_id . "'"
                             );

    // no comment history for this user & document
    if (!$dbresult) {
        return $toreturn;
    }

    // put the query result into an array
    for ($i = 0; $i < $dbresult->RecordCount(); $i++) {
        $toreturn[$dbresult->Fields('comment_id')] = $dbresult->Fields('visited');
        $dbresult->MoveNext();
    }

    //return an array with comment id's
    return $toreturn;
}


/**
 * This function removes the comment and its row from the db
 * It returns boolean value according to db-operation success.
 *
 * @param string $comment_id  ID of the comment to be marked read by the user
 * @return boolean true if operation successful false if operation failed
 */
function anno_remove($comment_id) {
    // Use the Moodle database connection
    global $db;

    //check the parameters
    if (!is_string($comment_id)) {
        return false;
    }

    // Add slashes to the comment ID
    $comment_id = addslashes($comment_id);

    //make sure there is something to delete
    $comment = $db->Execute("SELECT comment_id FROM anno_comment_history WHERE comment_id='$comment_id'");

    if($comment->fields[0] == $comment_id) {
        //comment found -> deleting
        $sql = "DELETE FROM anno_comment_history WHERE comment_id='$comment_id'";
        $db->Execute($sql);
        return true;
    } else {
        //comment was not in database -> can't be deleted
        return false;
    }
}

?>
