<?php

$path = "/home/tkt_ann3/public_html/moodle2/anno/handler_module";
set_include_path(get_include_path() . PATH_SEPARATOR . $path);

require_once "Anno_Stub.php";
require_once "PHPUnit/Framework.php";
require_once "anno_annotea_lib.php";
require_once(UTF8 . '/trim.php');

class AnnoteaLibTest extends PHPUnit_Framework_TestCase {

  protected $documentID;
  protected $text;
  protected $creatorID;
  protected $place;
  protected $category;
  protected $rootID;
  protected $parentID;
  protected $messageID;
  protected $commentid;

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 1  - a NULL value.
  // A negative testcase, should return true.
  public function test_anno_is_comment_locked_1() {
    $commentid = NULL;
    $expected_result = false; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 2  - an empty string.
  // A negative testcase, should return true.
  public function test_anno_is_comment_locked_2() {
    $commentid = "";
    $expected_result = false; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 3  - a correct value.
  // A positive testcase, this comment should be locked.
  public function test_anno_is_comment_locked_3() {
    $commentid = "0000000030";
    $expected_result = true; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 4  - an incorrect value, but correct format.
  // A non-existent commentid, should return true.
  public function test_anno_is_comment_locked_4() {
    $commentid = "00000FFFFF";
    $expected_result = true; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 5  - a correct value, but locked.
  // A positive testcase, this comment should not be locked.
  // This testcase requires special attention.
  // The comment needs to be created by this user just before
  // running this testcase.
  public function test_anno_is_comment_locked_5() {
    $commentid = "00000002DD";
    $expected_result = false; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 6  - value too long.
  // A negative testcase, should return true.
  public function test_anno_is_comment_locked_6() {
    $commentid = "000000003000000FFFFF";
    $expected_result = true; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 7  - value in wrong format.
  // A negative testcase, should return true.
  public function test_anno_is_comment_locked_7() {
    $commentid = "XXXX000030";
    $expected_result = true; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_is_comment_locked($commentid).
  // Testcase 8  - incorrect value type (integer).
  // A negative testcase, should return true.
  public function test_anno_is_comment_locked_8() {
    $commentid = 30;
    $expected_result = true; 
    $result = anno_is_comment_locked($commentid);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_has_replies($messageID).
  // Testcase 1 - a NULL value.
  // A negative testcase, should return false.
  public function test_anno_has_replies_1() {
    $messageID = NULL;
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_has_replies($messageID).
  // Testcase 2  - an empty string.
  // A negative testcase, should return false.
  public function test_anno_has_replies() {
    $messageID = "";
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_has_replies($messageID).
  // Testcase 3  - an incorrect value.
  // Incorrect value but correct format.
  // A non-existent messageID, should return false.
  public function test_anno_has_replies_3() {
    $messageID = "00000FFFFF";
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_has_replies($messageID).
  // Testcase 4  - a correct value.
  // Comment exists but has no replies.
  // This testcase requires manual steps of creating the comment.
  public function test_anno_has_replies_4() {
    $messageID = "0000000003";
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_has_replies($messageID).
  // Testcase 5  - a correct value.
  // Comment exists and should have replies.
  // This testcase requires manual steps of creating the comment and a reply.
  public function test_anno_has_replies_5() {
    $messageID = "00000002DD";
    $expected_result = true;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }




  /*
  // Testcase for function anno_has_replies($messageID).
  // Testcase 4  - a correct value.
  // Comment exists but has no replies.
  public function test_anno_has_replies_4() {
    // Let's create the comment first.
    $documentID = "http://testing.address.com/justfortesting.html";
    $text = "This is just a test comment.";
    $creatorID = "4: Miro W";
    $place = 1234;
    $category = "content";
    $commentid = anno_add_comment($documentID,$text,$creatorID,$place,$category);
    // commentid and messageID are basically the same variable
    $messageID = $commentid;
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_has_replies($messageID).
  // Testcase 5  - a correct value.
  // Comment exists and should have replies.
  public function test_anno_has_replies_5() {
    // Let's create the comment first.
    $documentID = "http://testing.address.com/justfortesting.html";
    $text = "This is just a test comment.";
    $creatorID = "4: Miro W";
    $place = 1222;
    $category = "content";
    $commentid = anno_add_comment($documentID,$text,$creatorID,$place,$category);
    // Let's create the reply to the comment.
    $rootID = $commentid;
    $text = "This is a test reply to a comment.";
    $creatorID = "4: Miro W";
    $parentID = $commentid;
    $messageID = anno_add_message($rootID,$text,$creatorID,$parentID);
    // Reply has been added.
    $expected_result = true;
    $result = anno_has_replies($commentid);
    $this->assertEquals($expected_result, $result);
  }
 */


  // Testcase for function anno_has_replies($messageID).
  // Testcase 6  - value in wrong format.
  // A negative testcase, should return false.
  public function test_anno_has_replies_6() {
    $messageID = "XXXX000030";
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_has_replies($messageID).
  // Testcase 7  - incorrect value type (integer).
  // A negative testcase, should return false.
  public function test_anno_has_replies_7() {
    $messageID = 30;
    $expected_result = false;
    $result = anno_has_replies($messageID);
    $this->assertEquals($expected_result, $result);
  }

}
?>
