<?php

$path = "/home/tkt_ann3/public_html/moodle2/anno/handler_module";
set_include_path(get_include_path() . PATH_SEPARATOR . $path);

require_once "Anno_Stub.php";
require_once "PHPUnit/Framework.php";
require_once "anno_handle_request.php";
require_once "anno_annotea_lib.php";


class HandleRequestTest extends PHPUnit_Framework_TestCase {

  // Testcase for function utf2html($str).
  // Testcase 1  - a NULL value.
  // A negative testcase, should return an empty string.
  public function test_utf2html_1() {
    $str = NULL;
    $expected_result = ""; 
    $result = utf2html($str);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function utf2html($str).
  // Testcase 2  - an empty string.
  // A negative testcase, should return an empty string.
  public function test_utf2html_2() {
    $str = "";
    $expected_result = ""; 
    $result = utf2html($str);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function utf2html($str).
  // Testcase 3  - a correct value.
  // Contains characters with ASCII value above 192.
  // A positive testcase, should return a correct 8-bit string.
  public function test_utf2html_3() {
    $str = utf8_encode("");
    $expected_result = "&#246;&#228;&#229;"; 
    $result = utf2html($str);
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function utf2html($str).
  // Testcase 4  - an incorrect value, but correct format.
  // Does NOT contain any characters with an ASCII value above 192.
  // Should return the same string.
  public function test_utf2html_4() {
    $str = "test string";
    $expected_result = "test string"; 
    $result = utf2html($str);
    $this->assertEquals($expected_result, $result);
  }

  /* Testcase commented out due to not testing a real functionality.
  // Testcase for function utf2html($str).
  // Testcase 5  - value in wrong format.
  // A negative testcase, the value is not in UTF-8 format.
  // Should return an empty string.
  public function test_utf2html_5() {
    $str = "";
    $expected_result = ""; 
    $result = utf2html($str);
    $this->assertEquals($expected_result, $result);
  }
  */

// ***************************************************************************************************************


  // Testcase for function anno_create_comment_views($comments, $rights, $userid, $documentid).
  // Testcase 1  - a NULL value.
  // The primary parameters for testing are $comments
  // A negative testcase, should return an empty string.
  public function test_anno_create_comment_views_1() {
    $str = NULL;
    $expected_result = "";
    $result = anno_create_comment_views($str,'','','');
    $this->assertEquals($expected_result, $result);
  }
 

 
// ***************************************************************************************************************

  // Testcase for function anno_create_multiview_pane(&$htmlpage).
  // Testcase 1  - a NULL value.
  // A negative testcase, should return an empty string.
  public function test_anno_create_multiview_pane_1() {
    $str = NULL;
    $expected_result = "";
    $result = anno_create_multiview_pane($str);
    $result=$str;
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_create_multiview_pane(&$htmlpage).
  // Testcase 2  - an empty string.
  // A negative testcase, should return an empty string.
  public function test_anno_create_multiview_pane_2() {
    $str = "";
    $expected_result = "";
    $result = anno_create_multiview_pane($str);
    $result=$str;
    $this->assertEquals($expected_result, $result);
  }
 
  // Testcase for function anno_create_multiview_pane(&$htmlpage).
  // Testcase 3 - a positive test case. 
  // Should return correctly formatted text.
  public function test_anno_create_multiview_pane_3() {
    $str = "</body>";
    $expected_result = file_get_contents('HandleRequestExpected.txt');
    $result = anno_create_multiview_pane($str);
    $result=$str;
    $this->assertEquals($expected_result, $result);
  }
 

  // Testcase for function anno_create_multiview_pane(&$htmlpage).
  // Testcase 4  - correct format but unexpected value.
  // A negative testcase, should return an empty string.
  public function test_anno_create_multiview_pane_4() {
    $str = "<body>";
    $expected_result = "<body>";
    $result = anno_create_multiview_pane($str);
    $result=$str;
    $this->assertEquals($expected_result, $result);
  }
 

// ***************************************************************************************************************
  
  // Testcase for function anno_add_comment_array(&$htmlpage, $commentarray).
  // Testcase 1  - a NULL value.
  // A negative testcase, should return an empty string.
  public function test_anno_add_comment_array_1() {
    $str = NULL;
    $expected_result = "";
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);

    anno_add_comment_array($str, $comarray);
    $this->assertEquals($expected_result, $str);
  }
  
  // Testcase for function anno_add_comment_array(&$htmlpage, $commentarray).
  // Testcase 2  - a NULL value.
  // A negative testcase, should return unmodified html-string
  public function test_anno_add_comment_array_2() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $expected_result = "".$str;
    $comarray[] = NULL;
    
    anno_add_comment_array($str, $comarray);
    $this->assertEquals($expected_result, $str);
  } 
  
  // Testcase for function anno_add_comment_array(&$htmlpage, $commentarray).
  // Testcase 3  - empty string.
  // A negative testcase, should return unmodified empty string.
  public function test_anno_add_comment_array_3() {
    $str = "";
    $expected_result = "";
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
    
    anno_add_comment_array($str, $comarray);
    $this->assertEquals($expected_result, $str);
  }

  // Testcase for function anno_add_comment_array(&$htmlpage, $commentarray).
  // Testcase 4  - incorrect format.
  // A negative testcase, should return unmodified string.
  public function test_anno_add_comment_array_4() {
    $str = "<html><body><div><p>A great test stub.</p></div></body></html>";
    $expected_result = "<html><body><div><p>A great test stub.</p></div></body></html>";
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
    
    anno_add_comment_array($str, $comarray);
    $this->assertEquals($expected_result, $str);
  }

  // Testcase for function anno_add_comment_array(&$htmlpage, $commentarray).
  // Testcase 5  - correct values.  
  // A positive testcase.
  public function test_anno_add_comment_array_5() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $expected_result = "<html><head>";
    $expected_result .= "\n<script language=\"JavaScript\" type=\"text/javascript\">";
    $expected_result .= "\n<!-- // Array of all the comments in this document";
    $expected_result .= "\nvar commentarray = new Array(2);";
    $expected_result .= "\ncommentarray[0] = new Array(2);";
    $expected_result .= "\ncommentarray[0][0] = \"id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \\\"\\\", 2, 5))\";";
    $expected_result .= "\ncommentarray[0][1] = \"content\";";
    $expected_result .= "\ncommentarray[1] = new Array(2);";
    $expected_result .= "\ncommentarray[1][0] = \"id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \\\"\\\", 5, 5))\";";
    $expected_result .= "\ncommentarray[1][1] = \"content\";";
    $expected_result .= "\n// done hiding -->";
    $expected_result .= "\n</script>";
    $expected_result .= "\n</head><body><div><p>A great test stub.</p></div></body></html>";
    
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
    
    anno_add_comment_array($str, $comarray);
    $this->assertEquals($expected_result, $str);
  }


}
?>
