<?php

$path = "/home/tkt_ann3/public_html/moodle2/anno/handler_module";
set_include_path(get_include_path() . PATH_SEPARATOR . $path);

require_once "Anno_Stub.php";
require_once "PHPUnit/Framework.php";
require_once "anno_x_pointer_lib.php";
require_once "anno_annotea_lib.php";


class XPointerLibTest extends PHPUnit_Framework_TestCase {

  // Testcase for function anno_do_overlapping_comment($document, $spannode, $tomarkread,
  //                  $comment, &$totallen, $params, &$counter, &$foundstart, &$foundend).
  // Testcase 1  - NULL.  
  // A negative testcase. Should leave $document unmodified (only dom additions).
  public function test_anno_do_overlapping_comment_1() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    $start = false;
    $end = false;
    $lngth = 2;

    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";   
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";   
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);

    $textnode = $domstub->createTextNode("spanned text");
    $spannode = anno_create_span_node($domstub, 1, $com2, $cntr);

    $paramstub = array("xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "string" => "",
                      "offset" => "2",
                      "length" => "5");

    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div><p>A great test stub.</p></div></body>\n</html>\n";

    $com1 = NULL;
    anno_do_overlapping_comment($domstub, $spannode, 1, $com1, $lngth, $paramstub, $cntr, $start, $end);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  } 

  // Testcase for function anno_do_overlapping_comment($document, $spannode, $tomarkread,
  //                  $comment, &$totallen, $params, &$counter, &$foundstart, &$foundend).
  // Testcase 2  - NULL.
  // A negative testcase. Should leave $document unmodified (only dom additions).
  public function test_anno_do_overlapping_comment_2() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    $start = false;
    $end = false;
    $lngth = 2;
   
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
   
    $textnode = $domstub->createTextNode("spanned text");
    $spannode = anno_create_span_node($domstub, 1, $com2, $cntr);
   
    $paramstub = array("xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "string" => "",
                      "offset" => "2",
                      "length" => "5");

    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div><p>A great test stub.</p></div></body>\n</html>\n";
    
    $spannode = NULL;
    anno_do_overlapping_comment($domstub, $spannode, 1, $com1, $lngth, $paramstub, $cntr, $start, $end);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  }


////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  // Testcase for function anno_do_stringrange($document, $tomarkread, $comment, $params, &$counter)
  // Testcase 1  - correct parameters
  // A positive testcase.
  public function test_anno_do_stringrange_1() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
    
    $paramstub = array("xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "string" => "",
                      "offset" => "2",
                      "length" => "5");
    
    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div><p>A ";
    $expected_result .= "<span id=\"anno_0000000028_1\" class=\"anno_highlight_content_read\" ";
    $expected_result .= "onmouseover=\"anno_on_mouse_over_comment(event, '0000000028');\">great</span> test stub.</p></div></body>\n</html>\n";
    
    anno_do_stringrange($domstub, 1, $com1, $paramstub, $cntr);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  }

  // Testcase for function anno_do_stringrange($document, $tomarkread, $comment, $params, &$counter)
  // Testcase 2  - NULL
  // A negative testcase.
  public function test_anno_do_stringrange_2() {
    $str = "<html><head></head><body><div><p>A great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";   
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 2, 5))";
    $com1->category = "content";
    $com2 = new AnnoMessageData;
    $com2->id = "0000000029";
    $com2->creator = "2:Admin User";
    $com2->date = "2007-05-05T03:27:15+03:00";
    $com2->content = "teksti";
    $com2->position = "http://testi.fi/view.php?id=5#xpointer(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 5, 5))";
    $com2->category = "content";
    $comarray = array($com1, $com2);
    
    $paramstub = array("xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "string" => "",
                      "offset" => "2",
                      "length" => "5");
  
    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div><p>A great test stub.</p></div></body>\n</html>\n";
    $com1 = NULL;
    
    anno_do_stringrange($domstub, 1, $com1, $paramstub, $cntr);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  }


////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    
  // Testcase for function anno_do_rangeto($document, $tomarkread, $comment, $params, &$counter)
  // Testcase 1  - correct parameters
  // A positive testcase.
  public function test_anno_do_rangeto_1() {
    $str = "<html><head></head><body><div><p>A great test stub.</p><p>Second great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";   
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(start-point(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 7, 1))";
    $com1->position .= "/range-to(end-point(string-range(//html[1]/body[1]/div[1]/p[2], \"\", 2, 1))))";
    $com1->category = "content";

    $fp = fopen("/var/tmp/tkt_ann3/debuglog", "a");
    fwrite($fp, date("[Y-m-d H:i:s]").": ".$com1->position."\n");
    fclose($fp);
   
    $paramstub = array("start_xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "start_string" => "",
                      "start_offset" => "7",
                      "start_length" => "1",
                         "end_xpath" => "//html[1]/body[1]/div[1]/p[2]",
                        "end_string" => "",
                        "end_offset" => "2",
                        "end_length" => "1");
  
    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div>\n";
    $expected_result .= "<p>A great<span id=\"anno_0000000028_1\" class=\"anno_highlight_content_read\" ";
    $expected_result .= "onmouseover=\"anno_on_mouse_over_comment(event, '0000000028');\"> test stub.</span></p>\n";
    $expected_result .= "<p><span id=\"anno_0000000028_2\" class=\"anno_highlight_content_read\" ";
    $expected_result .= "onmouseover=\"anno_on_mouse_over_comment(event, '0000000028');\">Se</span>cond great test stub.</p>\n";
    $expected_result .= "</div></body>\n</html>\n";
    
    anno_do_rangeto($domstub, 1, $com1, $paramstub, $cntr);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  }


  // Testcase for function anno_do_rangeto($document, $tomarkread, $comment, $params, &$counter)
  // Testcase 2  - NULL
  // A negative testcase.
  public function test_anno_do_rangeto_2() {
    $str = "<html><head></head><body><div><p>A great test stub.</p><p>Second great test stub.</p></div></body></html>";
    $domstub = new DOMDocument();
    @$domstub->loadHTML($str);
    $cntr = 1;
    
    $com1 = new AnnoMessageData;
    $com1->id = "0000000028";
    $com1->creator = "2:Admin User";
    $com1->date = "2007-05-06T02:30:06+03:00";
    $com1->content = "juttu";
    $com1->position = "http://testi.fi/view.php?id=5#xpointer(start-point(string-range(//html[1]/body[1]/div[1]/p[1], \"\", 7, 1))";
    $com1->position .= "/range-to(end-point(string-range(//html[1]/body[1]/div[1]/p[2], \"\", 2, 1))))";
    $com1->category = "content";
   
    $fp = fopen("/var/tmp/tkt_ann3/debuglog", "a");
    fwrite($fp, date("[Y-m-d H:i:s]").": ".$com1->position."\n");
    fclose($fp);
    
    $paramstub = array("start_xpath" => "//html[1]/body[1]/div[1]/p[1]",
                      "start_string" => "",
                      "start_offset" => "7",
                      "start_length" => "1",
                         "end_xpath" => "//html[1]/body[1]/div[1]/p[2]",
                        "end_string" => "",
                        "end_offset" => "2",
                        "end_length" => "1");
    
    $com1 = NULL;
    $expected_result = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    $expected_result .= "<html>\n<head></head>\n<body><div>\n";
    $expected_result .= "<p>A great test stub.</p>\n<p>Second great test stub.</p>\n";
    $expected_result .= "</div></body>\n</html>\n";
    
    anno_do_rangeto($domstub, 1, $com1, $paramstub, $cntr);
    $result = $domstub->saveHTML();
    $this->assertEquals($expected_result, $result);
  }


}
?>

