/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/** 
 * Functions to show the comment view 'tooltip'.
 *
 * @author  Esko Lehtonen, Jarno Laitinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and   
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 * 
 */


/**
 * Open a new comment view.
 * 
 * @param anno_current_tooltip 
 * @param targetElement Target element
 * @return this
 */
var anno_closingtime;
var timerNotOn = true;

function anno_new_comment_view(anno_current_tooltip, targetElement) {
    this.anno_current_tooltip = anno_current_tooltip;

    this.targetElement = targetElement; //comment span element that activated new comment view

    // Event handler to hide the tooltip 
    this.mmb = function(event) { 
		if (!event) 
			var event = window.event;
		if(event.type == "mouseout" && timerNotOn) {
            anno_closingtime=setTimeout("this.anno_deactivate_current_tooltip()", 2000);
            timerNotOn = false;
        }
        if(event.type == "mouseover") {
            clearTimeout(anno_closingtime); //stops timer
            timerNotOn = true;
        }
    }
	
    // Even handler to catch event from bubbling.
    this.ce = function(event) {        
        if(window.event) {  
            event.cancelBubble = true;
        } else {
            event.stopPropagation();
        }
    }
    
	
    // Called when this comment view is set visible
    function anno_activate_current_tooltip() {
        // W3C / Mozilla implemention
        if (this.anno_current_tooltip.addEventListener) {
            try {
                // Add the closing event handler to the body after one second delay.
                // Delay is important to allow the user to move the pointer over the comment
                // view if it is not over it yet.
                //clearTimeout(anno_closingtime); //stops timer				
                this.anno_current_tooltip.addEventListener('mouseover', this.mmb, false);
                this.anno_current_tooltip.addEventListener('mouseout', this.mmb, false);

                // Add event handler to catch all mousemove event. This is used to 
                // prevent bubbling of the events from inside of the comment view to the outside.
                // If this would not be here, the closing event handler would catch the event
                // and close the comment view even if the pointer would be on the comment view.
                this.anno_current_tooltip.addEventListener('mouseover', this.ce, false);
                this.anno_current_tooltip.addEventListener('mouseout', this.ce, false);
                this.targetElement.addEventListener('mouseover', this.mmb, false);
                this.targetElement.addEventListener('mouseout', this.mmb, false);
                this.targetElement.addEventListener('mouseover', this.ce, false);
                this.targetElement.addEventListener('mouseout', this.ce, false);
            } catch(e) {
                document.write(this + ": "+ this.anno_current_tooltip +" : show <br/ >");
            }
        }

        // IE implementation
        if (this.anno_current_tooltip.attachEvent) {
            try {
                this.anno_current_tooltip.attachEvent('onmouseover', this.mmb);
                this.anno_current_tooltip.attachEvent('onmouseout', this.mmb);
                this.anno_current_tooltip.attachEvent('onmouseover', this.ce);
                this.anno_current_tooltip.attachEvent('onmouseout', this.ce);
                this.targetElement.attachEvent('onmouseover', this.mmb);
                this.targetElement.attachEvent('onmouseout', this.mmb);
                this.targetElement.attachEvent('onmouseover', this.ce);
                this.targetElement.attachEvent('onmouseout', this.ce);
            } catch(e) {
                document.write(this + ": "+ this.anno_current_tooltip +" : show <br/ >"); 
            }
        }
        
        // set the comment view visible
        this.anno_current_tooltip.style.display = "block";   
    }
	
    this.activate = anno_activate_current_tooltip;
    this.deactivate = anno_deactivate_current_tooltip;    
    
    return this;
}


/**
 * Called when this comment view is hidden
 */
function anno_deactivate_current_tooltip() {
    // W3C / Mozilla implemenation
    clearTimeout(anno_closingtime); //stops timer
    timerNotOn = true;
    if (this.anno_current_tooltip.removeEventListener) {
        try {
             this.anno_current_tooltip.removeEventListener('mouseover', this.mmb, false);
             this.anno_current_tooltip.removeEventListener('mouseout', this.mmb, false);	
             this.anno_current_tooltip.removeEventListener('mouseover', this.ce, false);
             this.anno_current_tooltip.removeEventListener('mouseout', this.ce, false);
             this.targetElement.removeEventListener('mouseover', this.mmb, false);
             this.targetElement.removeEventListener('mouseout', this.mmb, false);
             this.targetElement.removeEventListener('mouseover', this.ce, false);
             this.targetElement.removeEventListener('mouseout', this.ce, false);			
        } catch (e) {
             document.write(e + "hide <br />");
        }
    } else if (this.anno_current_tooltip.detachEvent) {
        try {
             this.anno_current_tooltip.detachEvent('onmouseover', this.mmb);
             this.anno_current_tooltip.detachEvent('onmouseout', this.mmb);
             this.anno_current_tooltip.detachEvent('onmouseover', this.ce);
             this.anno_current_tooltip.detachEvent('onmouseout', this.ce);
             this.targetElement.detachEvent('onmouseover', this.mmb);
             this.targetElement.detachEvent('onmouseout', this.mmb);
             this.targetElement.detachEvent('onmouseover', this.ce);
             this.targetElement.detachEvent('onmouseout', this.ce);
        } catch (e) {
             document.write(e + "hide <br />");
        }
    }

    // hide the tooltip
    this.anno_current_tooltip.style.display = "none"; 
    //anno_closingtime=setTimeout("this.anno_deactivate_current_tooltip()", 0);
	
}
