/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Functions for using XMLHttpRequest to pass data to the Anno server-side code.
 *
 * @author Paula Urpilainen, Heikki Aitakangas
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 */


/**
 * Creates a XMLHttpRequest object, with handling for different browsers.
 *
 * @return The XMLHttpRequest object created or false on failure.
 */
function anno_create_XMLHttpRequest() {
    var oRequest = false;
    if (window.XMLHttpRequest) { // Mozilla, Opera, Safari,...
        try {
            oRequest = new XMLHttpRequest();
            // Override the server-provided MIME-type. Technically unnecessary, if we do things right on server side
            if (oRequest.overrideMimeType) {
                oRequest.overrideMimeType('text/xml');
            }
        } catch (e) {
            oRequest = false;
        }
    } else if (window.ActiveXObject) { // IE
        try {
            oRequest = new ActiveXObject("Msxml2.XMLHTTP");
        } catch (e) {
            try {
                oRequest = new ActiveXObject("Microsoft.XMLHTTP");
            } catch (e) {
                oRequest = false;
            }
        }
    } else {
        oRequest = false;
    }

    return oRequest;
}


/**
 * Encodes the given given array for use as POST data.
 * Array contents should be of form
 * [[paramname], [paramdata],
 *  [paramname], [paramdata],
 *  [paramname], [paramdata],
 *            ...           ]
 *
 * @param aData  The array whose contents to encode
 */
function anno_encode_post_data(aData) {
    var sReturn = '';
    for(var i = 0; i < aData.length; i++) {
        // If it's a parameter other than the first, add a preceding &
        if(i != 0) {
            sReturn += '&';
        }
        // Encode the parameter
        /// @todo Test this in IE
        sReturn += encodeURIComponent(aData[i][0]) + '=' + encodeURIComponent(aData[i][1]);
    }

    return sReturn;
}


/**
 * Sends a request to anno_server_uri with the given method and data.
 * For POST requests (only request type supported atm, anyway :P),
 * the data should be formatted using anno_encode_post_data().
 *
 * This is a synchronous request, which may have performance implications.
 *
 * @param sMethod The connection method, as string. "GET", "POST", etc...
 * @param sData   The data to be sent in the message.
 * @return Either the connection object on success or false on failure.
 */
function anno_send_request(sMethod, sData) {
    // Init the request object that will handle this request
    var oRequest = anno_create_XMLHttpRequest();
    if (!oRequest) {
        document.getElementById("anno_errortext").innerHTML= "Yhteys palvelimeen ei toimi.<br/>"
                    + "Syy: XmlHttpRequestin luominen ep\xe4onnistui.<br/>"
                    + "K\xe4yt\xe4t ehk\xe4 selainta, joka ei ole Anno-annotointity\xf6kalun kanssa yhteensopiva.";
        document.getElementById("anno_errorview").style.visibility = 'visible';
        return false;
    }

    // Send the request synchronously
    try {
        oRequest.open(sMethod, anno_server_uri, false);
        oRequest.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
        oRequest.send(sData);
    }
    // Error handling, alert if it's a messed up server address. In any case, return false to indicate failure
    catch (e) { 
        document.getElementById("anno_errortext").innerHTML= "Yhteys palvelimeen ei toimi.<br/>"
        + "Selaimen virheilmoitus: " + e; 
        document.getElementById("anno_errorview").style.visibility = 'visible';
        return false; 
    } 

    if (oRequest === false) { 
        document.getElementById("anno_errortext").innerHTML= "Yhteys palvelimeen ei toimi.<br/>"
                    + "Syy: XmlHttpRequestin luominen ep\xe4onnistui.<br/>"
                    + "K\xe4yt\xe4t ehk\xe4 selainta, joka ei ole Anno-annotointity\xf6kalun kanssa yhteensopiva.";
        document.getElementById("anno_errorview").style.visibility = 'visible';
        return false; 
    } else if (oRequest.status != 200) { 
        document.getElementById("anno_errortext").innerHTML= "Toiminto ep\xe4onnistui:<br/> "+ oRequest.responseText;
        document.getElementById("anno_errorview").style.visibility = 'visible';
        return false; 
    } 
    return oRequest;
}


/**
 * Sends a request to anno_server_uri with the given method and data.
 * POST data should be encoded using anno_encode_post_data().
 *
 * This is an asynchronous request, meaning it will return directly and
 * you callback function will get called once the request has been
 * completed successfully
 *
 * @param sMethod   The connection method, as string. "GET", "POST", etc...
 * @param sData     The data to be sent in the message.
 * @param fCallback The function to be called when the request is completed.
 *                  Should take one parameter, which will be the XMLHttpRequest
 *                  object used to make the request.
 * @return Whether we managed to initiate the request. True if we did, false
 *         we failed.
 */
function anno_send_request_callback(sMethod, sData, fCallback) {
    // Init the request object that will handle this request
    var oRequest = anno_create_XMLHttpRequest();
    if (!oRequest) {
        /// @todo Handle error
        return false;
    }

    // Set up the ready state change handler
    oRequest.onreadystatechange =
        function (event){
            // See if we reached the final state
            if (oRequest.readyState == 4) {
                fCallback(oRequest);
            }
        }

    // Send the request asynchronously
    try {
        oRequest.open(sMethod, anno_server_uri, true);
        oRequest.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
        oRequest.send(sData);
    }
    // Error handling, alert if it's a messed up server address. In any case, return false to indicate failure
    catch (e ){
        if (e == 'Permission denied to call method XMLHttpRequest.open') {
            alert(e + "\nBug in the server address, it's something that Javascript is not allowed to access:\n" + anno_server_uri);
        }
        return false;
    } 
    // Notify success
    return true;
}
