/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Event handler function for the ui module.
 *
 * @author  Mika Holmstrm, Esko Lehtonen, Jarno Laitinen, Miro Wikgren
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */


/**
 * Shows the comment view when the user points a comment marked
 * on the document. Also marks the comment as read.
 *
 * @param event event
 * @param comment_id Id of the comment to be viewed. 
 */
function anno_on_mouse_over_comment(event, commentId) {

    // Prevent event bubbling
    if (!event) {
        var event = window.event;
    }
	var target = anno_util_GetEventElement(event);
	var x; var y;
	  
    if (target.className.indexOf("anno_message_")!=-1) {
        var pos = findPos(document.getElementById("anno_comment_view_multiview_text"));
        x = pos.x;
        y = pos.y;
    } else {
        var pos = anno_util_positionWindow(event, "singleview");
        x = pos.x;
        y = pos.y;
    }
	
    event.cancelBubble = true;
    if (event.stopPropagation) {
        event.stopPropagation();
    }

    var txt = ''; 
    if (window.getSelection) { // Mozilla/Firefox, Safari
        txt = window.getSelection(); 
    } else if (document.getSelection) { // Explorer 5+ Mac,
                                        // Mozilla/Firefox,
                                        // Opera, Netscape 
        txt = document.getSelection(); 
    } else if (document.selection) { // Explorer 5+ Win
        txt = document.selection.createRange().text;
    } 
    if (txt!='') {
        return true; 
    }

    /**
     * Find the coordinates for an element on the page
     */
    function findPos(obj) {
        var curleft = curtop = 0;
        if (obj.offsetParent) {
            curleft = obj.offsetLeft
            curtop = obj.offsetTop
            while (obj = obj.offsetParent) {
                curleft += obj.offsetLeft
                curtop += obj.offsetTop
            }
        }
        return {x: curleft, y: curtop};
    }

    // mark it as read
    anno_mark_comment_read(commentId);
	
    

    // actully open the comment view on specific place
    anno_spawn_comment_view(x, y, commentId, target); // in anno_pop_up.js
}


/**
 * Handles the mouseOver event for spans where we have several comments on top of each other.
 *
 * @param   event      This is the javascript event object. 
 * @param   spanID     ID of the element that generated the event.
 */
var anno_last_multiview_id;
var lastx; var lasty;

function anno_on_mouse_over_comment_multiple(event, spanID) {

    // This prevents the event from propagating (bubbling) to ancestor elements.
    // This means that mouseOver handler will only be caller for the innermost span elent.
    if (!event) {
        var event = window.event;
    }
    event.cancelBubble = true;
    if (event.stopPropagation) {
        event.stopPropagation();
    }

    // save the spanID for later, so we can reopen the view if we want
    anno_last_multiview_id=spanID; 
         
    var txt = '';
    if (window.getSelection) { // Mozilla/Firefox, Safari
        txt = window.getSelection(); 
    } else if (document.getSelection) { // Explorer 5+ Mac,
                                        // Mozilla/Firefox,
                                        // Opera, Netscape 
        txt = document.getSelection(); 
    } else if (document.selection) { // Explorer 5+ Win
        txt = document.selection.createRange().text; 
    } 
    if (txt!='') {
        return true;
    }

    // DO NOT mark anything as read
    // The spanID looks like "00001df_0000163_000014a". 
    var IDs=spanID.split("_");

    // Fill the view pane with information for each comment
    var comments="Valitse kommenttiketju:" ;
    for (var n=0;n<IDs.length;n++) { 
        comments = comments + "<table class=\"anno_message_content " +
document.getElementById("anno_comment_style_"+IDs[n]).className +
"\" onclick=\"anno_on_mouse_over_comment(event, '" +                                                                                            
IDs[n] + "');anno_stop_highlighting();document.getElementById('anno_return_to_multiple_" + IDs[n]+ "').style.visibility='visible';\" onmouseover=\"anno_highlight_comment('" +
IDs[n] + "');\" onmouseout=\"anno_stop_highlighting();\"><tr><td class=\"anno_message_author_created\">"
+ document.getElementById("anno_message_author_created_" + IDs[n]).innerHTML +
"</td><td></td></tr><tr><td colspan=2 class=\"anno_message_text\">" +
document.getElementById("anno_message_text_"+IDs[n]).innerHTML + "</td></tr></table>";
    }

    document.getElementById("anno_comment_view_multiview_text").innerHTML=comments;

    var x; var y;
    var target = event.target ? event.target : event.srcElement;
    if (target.className.indexOf("anno_return_to_multiple")!=-1) {
        x = lastx; 
        y = lasty;
    } else { 
        var pos = anno_util_positionWindow(event, "multiview");
        x = pos.x;
        y = pos.y;
        // actully open the comment view on specific place 
    } 
    lastx=x; lasty=y; 

    // show the view pane
    targetElement = anno_util_GetEventElement(event);
    anno_spawn_comment_view(x, y, "multiview", targetElement); // in anno_pop_up.js
}


/**
 * Highlights all comment span tags which have the specified ID.
 * 
 * @param commentID comment id
 */
var highlightedElement; // an array which stores references to the highlighted elements, so that their style can be reset.
var originalStyle; // an array with the original styles of the highlighted elements, so they can be reset in anno_stop_highlighting().
function anno_highlight_comment(commentID) {
    var e=document.getElementsByTagName("span");
    highlightedElement = Array();
    originalStyle = Array();
    var j=0;
    for(var i=0;i<e.length;i++) {
        if (e[i].id.indexOf(commentID)!=-1) {
            highlightedElement[j]=e[i];
            originalStyle[j]=e[i].style.backgroundColor;
            e[i].style.backgroundColor="yellow";
            j++;
        }
    }
}


/**
 * Removes highlighting from all highlighted span tags.
 */
function anno_stop_highlighting() {
    for(var i=0;i<highlightedElement.length;i++) {
        highlightedElement[i].style.backgroundColor=originalStyle[i];
    }
}


/**
 * Closes the view pane when the user explicitely
 * requests it.
 */
function anno_close_view_pane_new_comment() {
    anno_set_view_pane_new_comment_visible(false);
}


/**
 * Opens the help
 */
function anno_on_click_help() {
    window.open(anno_help_uri);
}


/**
 * Asks if a user really wants to delete comment/reply and
 * then does it.
 * 
 * @param event event
 * @param to_delete_id comment id
 */
function anno_on_click_delete(event, to_delete_id) {
    if (window.confirm("Haluatko varmasti poistaa kommentin?")) {
        anno_delete_comment(to_delete_id);
    }
}


/**
 * Opens the view pane modification mode.
 * If root_id == target_id, then the new message will be parent of the
 * root message.
 * 
 * @param string root_id The comment (in the document): root of the discussion
 * @param string target_id The comment to reply: the parent of the new message.
 */
function anno_on_click_edit(root_id, target_id) {
    var check = false;
    if (check) { 
        window.confirm("Palvelin ei ole viel ksitellyt edellist pyynt loppuun. Haluatko kuitenkin jatkaa?");
    }
    anno_set_current_root(root_id);
    anno_set_current_reply_target(null);
    anno_set_modification_target(target_id);
}


/** 
 * Called when the user clicks publish/add comment button from the view pane.
 * The behaviour depends on the values of certain global variables:
 * - anno_current_root_id 
 * - ANNO_CURRENT_REPLY_TARGET_ID
 * - anno_modification_target_id
 *  
 * These are used to determine wheather the user intends to add 
 * a new comment, to make a reply to the comment, or to modify the comment.
 */
function anno_on_click_publish() {
    //so you cant make two exactly the same comments
    anno_disableButton("anno_view_pane_button");
    anno_disableButton("Submit");

    // get the text from the textarea
    var textarea = document.getElementById("anno_new_comment_textarea");
    var text = textarea.value;
        
    // decide what we are doing here... 
    if (anno_current_root_id != null 
        && anno_current_reply_target_id != null) {    
        // we are going to make a response   
        
        // The text needs to be fetched from another element 
        var textarea = document.getElementById("anno_new_reply_textarea");
        text = textarea.value;

        // empty message are not allowed
        if (text == '' || text == null) {
            alert("Viesti ei voi olla tyhj\xe4");   
        } else {
            anno_respond_comment(anno_current_root_id, anno_current_reply_target_id, text);
        }
         
    } else if (anno_current_root_id != null
        && anno_modification_target_id != null) { 
        // we are going to modify the comment
        
        // if text is empty and this is not comment but message, then alert
        if ((text == '' || text == null) && (anno_current_root_id != anno_modification_target_id)) {
            alert("Viesti ei voi olla tyhj\xe4");   
        } else {
           anno_modify_comment(anno_current_root_id, anno_modification_target_id, text); 
        }
         
    } else if (anno_current_xpointer) {
        if (anno_current_comment_type != null) {
            // we are going to make a brand new comment
            // NB. Empty comments are allowed
            anno_add_comment(anno_current_xpointer, text, anno_current_comment_type);
        }
    }
    anno_util_enableButton("anno_view_pane_button");
    anno_util_enableButton("Submit");
}


/**
 * This is used to capture the html element where
 * mouse was pressed down, hence the name!
 * 
 * @param event event
 */
function anno_on_mouse_down_document(event) {
    if (!event) { 
        var event = window.event;
    }
    
    if (anno_cv != null) {   
        anno_cv.deactivate();
    }    
}

/**
 * Handle mouse up event.
 * 
 * @param event event
 */
function anno_on_mouse_up_document(event) {
    if (!event) {
        var event = window.event;
    }

    var geEndElement = event.target ? event.target : event.srcElement;

    anno_current_xpointer = sGetXPointerFromCurrentSelection(geEndElement);

    // If selection is empty (returned xpointer is null),
    // don't do anything, because user just clicked on the document.
    if (anno_current_xpointer) {
        // Show the comment category selector
        var selector = document.getElementById("anno_comment_type_selector");
        var pos = anno_util_positionWindow(event, "addComment");
        var x = pos.x; 
        var y = pos.y;
		
        selector.style.top = y + "px";
        selector.style.left = x + "px";
        selector.style.display = "block";
    }
}


/**
 * Shows comment type selection window.
 * 
 * @param type comment type
 */
function anno_on_click_comment_type(type) {
    // Close other views
    anno_set_view_pane_new_comment_visible(false);
    anno_set_view_pane_big_visible(false);
    anno_set_view_pane_reply_visible(false);

    var selector = document.getElementById("anno_comment_type_selector");
    selector.style.display = "none";

    anno_current_comment_type = type;

    var text = '';

    if (type == 'content') {
        text = "Kirjoita uusi sis\xe4lt\xf6kommentti";
    } else {
        text = 'Kirjoita uusi oikeinkirjoituskommentti';
    }

    var description = document.getElementById("anno_new_comment_description");
    description.lastChild.nodeValue = text;

    anno_set_current_root(null);
    anno_set_current_reply_target(null);
    anno_set_modification_target(null);

    // refresh the view pane
    anno_set_view_pane_new_comment_visible(true);
}


/**
 * Opens a reply view to the bottom banner.
 * 
 * @param root_id   root id
 * @param target_id target id
 */
function anno_on_click_respond(root_id, target_id) {
    anno_set_current_root(root_id);
    anno_set_current_reply_target(target_id);

    //anno_set_modification_target(null);

    // Close other views
    anno_set_view_pane_new_comment_visible(false);
    anno_set_view_pane_big_visible(false);

    anno_set_view_pane_reply_visible(true);
}


/**
 * Shows a selected comment tree in the bottom banner
 * 
 * @param root_id   root id
 */
function anno_on_click_show_big_view(rootid) {
    // Close all other views
    anno_set_view_pane_new_comment_visible(false);
    anno_set_view_pane_reply_visible(false);

    anno_set_current_root(rootid);

    // And show the big view
    anno_set_view_pane_big_visible(true);
}


/**
 * Closes the big window
 */
function anno_close_view_pane_big() {
    anno_set_view_pane_big_visible(false);
}


/**
 * Closes the reply window
 */
function anno_close_view_pane_reply() {
    anno_set_view_pane_reply_visible(false);
}


/**
 * Closes comment type selector window
 */
function anno_on_click_close_comment_type_selector() {
    var selector = document.getElementById("anno_comment_type_selector");
    selector.style.display = "none";   
}


/**
 * This is called when the use switches annotability on/off.
 *
 * @param boolean status If true, set annotation on. If false, set it off.
 */
function anno_on_click_toggle_annotation(status) {
    anno_toggle_is_annotable(status);
}
