/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Library function for the ui module.
 *
 * @author  Esko Lehtonen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */

/**
 * Checks if there's some anno specific parameters in the request.
 * These parameters are used to determine wheather to show
 * view pane and/or error message.
 *
 * Called by body onload eventhandler.
 */
var anno_requestSent;

function anno_init() {
    // first we parse paramater names and their values
    var search = window.location.search.substring(1);
    if (search.length > 0) {
        var params=search.split('&');
        for (var i=0 ; i<params.length ; i++) {
            var pos = params[i].indexOf("=");
            var name = params[i].substring(0, pos);
            var value = params[i].substring(pos + 1);

            // check if the parameter name matches

            // check for parameters to set global variables
            if (name == "anno_current_root_id") {
                anno_set_current_root(value);
            } else if (name == "anno_current_reply_target_id") {
                anno_set_current_reply_target( (value == '' ? null : value) );
            } else if (name == "anno_modification_target_id") {
                anno_set_modification_target( (value == '' ? null : value) );
            }
        }
    }
}


/**
 * Makes synchronous request to add a new comment to the database.
 *
 * @param   selection   XPointer to the target of the comment
 * @param   text        Content of the  comment
 * @param   category    Cateogory of the comment
 */
function anno_add_comment(selection, text, category) {
    // give immeadiate feedback to the user: the request may take some time
    anno_set_indicator("Tallennetaan kommenttia...");

    var http_method = "POST";
    var sData = anno_encode_post_data([['action', 'new_comment'],
                                        ['place', selection],
                                        ['text', text],
                                        ['category', category],
                                        ['document_id', anno_get_document_id()]]);

    // if in test mode, just log and reload
    if (anno_test_mode) {
        anno_test_log(sData);
        anno_reload_document();
    } else {
        var oRequest = anno_send_request(http_method, sData);
        if (oRequest!=false) {
            anno_reload_document();
        }
    }

    // hide the indicator
    anno_set_indicator(null);
}


/**
 * Makes synchronous request for adding a response to a comment.
 *
 * @param   root_id     Id of the comment in question.
 * @param   target_id   Id of the comment/message to be replied.
 *                      root_id and target_id are the same, if we are
 *                      replying directly to the comment.
 * @param   text        Content of the response.
 */
function anno_respond_comment(root_id, target_id, text) {
    // give immeadiate feedback to the user: the request may take some time
    anno_set_indicator("Tallennetaan vastausta...");

    var http_method = "POST";
    var sData = anno_encode_post_data([['action', 'reply'],
                                        ['root_id', root_id],
                                        ['mother_id', target_id],
                                        ['text', text],
                                        ['document_id', anno_get_document_id()]]);

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log(sData);
        anno_reload_document();
    } else {
        var oRequest = anno_send_request(http_method, sData);
        if (oRequest!=false) {
            anno_reload_document();
        }
    }

    // clear the indicator
    anno_set_indicator(null);
}


/**
 * Makes a syncronous request to modify content of the comment or message.
 *
 * @param   root_id         Id of the comment in question.
 * @param   to_modify_id    Id of the comment/message to be modified.
 *                          root_id and target_id are the same, if we are
 *                          mofiying the comment..
 * @param   text            New content of the comment/message.
 *
 */
function anno_modify_comment(root_id, to_modify_id, text) {
    // give immeadiate feedback to the user: the request may take some time
    anno_set_indicator("Tallennetaan muokkausta...");

    var http_method = "POST";
    if (root_id == to_modify_id) {
        // root_id == to_modify_id so this is a comment
        //
        var sData = anno_encode_post_data([['action', 'modify'],
                                            ['type', 'comment'],
                                            ['id', to_modify_id],
                                            ['comment_id', to_modify_id],
                                            ['message_id', to_modify_id],
                                            ['text', text],
                                            ['document_id', anno_get_document_id()]]);
    } else {
        // this is a message (= reply to a comment/message)
        var sData = anno_encode_post_data([['action', 'modify'],
                                            ['type', 'message'],
                                            ['id', to_modify_id],
                                            ['comment_id', to_modify_id],
                                            ['message_id', to_modify_id],
                                            ['text', text],
                                            ['place', null],
                                            ['document_id', anno_get_document_id()]]);
    }

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log(sData);
        anno_reload_document();
    } else {
        var oRequest = anno_send_request(http_method, sData);
        if (oRequest!=false) {
            anno_reload_document();
        }
    }
    // clear the indicator
    anno_set_indicator(null);
}


/**
 * Makes a request to delete a comment or message.
 *
 * @param   to_delete_if    The id of the comment or message to be deleted.
 */
function anno_delete_comment(to_delete_id) {
    // give immeadiate feedback to the user: the request may take some time
    anno_set_indicator("Poistetaan...");

    var http_method = "POST";
    var sData = anno_encode_post_data([['action', 'delete'],
                                        ['id', to_delete_id],
                                        ['document_id', anno_get_document_id()]]);

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log(sData);
        anno_reload_document();
    } else {
        var oRequest = anno_send_request(http_method, sData);
        if (oRequest!=false) {
            anno_reload_document();
        }
    }

    // clear the indicator
    anno_set_indicator(null);
}


/**
 * Makes a request to change the annotability of the current document.
 *
 * @param   bStatus     The desired status of the annotability.
 *                      True = annotability will be set on.
 *                      False = annotability wiil be set off.
 */
function anno_toggle_is_annotable(bStatus) {
    // give immeadiate feedback to the user: the request may take some time
    anno_set_indicator("Kytketään annotointia " + (bStatus ? 'päälle...' : 'pois') );

    var http_method = "POST";
    if (bStatus) {
        var sData = anno_encode_post_data([['action', 'set_annotation_on'],
                                            ['document_id', anno_get_document_id()]]);
    } else {
        var sData = anno_encode_post_data([['action', 'set_annotation_off'],
                                            ['document_id', anno_get_document_id()]]);
    }

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log(sData);
    } else {
        var oRequest = anno_send_request(http_method, sData);
        if (oRequest!=false) {
            anno_reload_document();
        }
    }

    // clear the indicator
    anno_set_indicator(null);
}


/**
 * Marks a comment as read on the document and makes an asyncronous request 
 * to save its status to the database.
 *
 * @param   comment_id  The element to be marked as read by color.
 * @param   root_id     The id of the comment to be marked as read.
 */
function anno_mark_comment_read(comment_id) {
    // Set comments as read on the document, basically, set their style
    // Because one comment may have multiple spans we first search
    // all the span-elements throught.

    // get alls spans
    // if the id matches, then set it's color
    var spans = document.getElementsByTagName('span');
    var regex = new RegExp(anno_comment_selection_prefix + comment_id);
    for (var i=0; i<spans.length; i++) {
        if (spans[i].id) {
            // if the comment_id matches... there may be counters at the end of the id
            // if the comment is marked with multiple spans
            if (regex.test(spans[i].id)) {
                anno_util_changeReadCSS(spans[i]);
            }
        }
    }
    // change the style for the multiview pane
    anno_util_changeReadCSS(document.getElementById("anno_comment_style_" + comment_id)); 
	
    var http_method = "POST";
    var sData = anno_encode_post_data([['action', 'mark_read'],
                                        ['id', comment_id],
                                        ['document_id', anno_get_document_id()]]);

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log(sData);
    } else {
        anno_send_request_callback(http_method, sData, function(oRequest){ });
    }
	//document.getElementById("anno_comment_style_" + comment_id).focus();
}


/**
 * Shows an indicator to notify user about ongoing transaction with the server.
 *
 * @param   status  Text to display on the indicator or null. If null, the
 *                  indicator will be hidden.
 */
function anno_set_indicator(status) {
    var indicator = document.getElementById("anno_indicator");
    if (status != null) {
        indicator.style.visibility = "visible";
        indicator.innerHTML = status;
    } else {
        indicator.style.visibility = "hidden";
    }
}


/**
 * Sets global variable anno_current_reply_target_id to the value given.
 * Checks that the id refers to some existent comment.
 *
 * @param   target_id   The id of the new reply target. If null, the variable
 *                      will be cleared.
 */
function anno_set_current_reply_target(target_id) {
    var e = document.getElementById(anno_comment_view_prefix + target_id);
    if (target_id == null || e != null) {
        anno_current_reply_target_id = target_id;
    }
}


/**
 * Sets global variable anno_current_root_id to the value given.
 * Checks that the id refers to some existent comment.
 *
 * @param   root_id     The id of the new root. If null, the variable will be
 *                      cleared.
 */
function anno_set_current_root(root_id) {
    var e = document.getElementById(anno_comment_view_prefix + root_id);
    if (root_id == null || e != null) {
        anno_current_root_id = root_id;
    }
}


/**
 * Sets global variable anno_modification_target_id to the value given.
 * Checks that the id refers to some existent comment.
 *
 * @param   target_id   The id of the new modification target. If null,
 *                      the variable will be cleared.
 */
function anno_set_modification_target(target_id) {
    var e = document.getElementById(anno_comment_view_prefix + target_id);
    if (target_id == null || e != null) {
        anno_modification_target_id = target_id;
    }
}


/**
 * Returns the document identifier of the current document.
 * Identifier is the URL without any anno specific paramters.
 * Anno specific parematers are anno_current_root_if,
 * anno_current_reply_target_id, anno_modification_target_id and
 * anno_error_msg.
 *
 * @return document_id document identifier of the current document
 */
function anno_get_document_id() {
    var document_id = window.location.href;
    document_id = document_id.replace(/&?anno_current_root_id=[^&]*&?/, '');
    document_id = document_id.replace(/&?anno_current_reply_target_id=[^&]*&?/, '');
    document_id = document_id.replace(/&?anno_modification_target_id=[^&]*&?/, '');
    document_id = document_id.replace(/#.*/, '');
    return document_id;
}


/**
 * Reloads the page using GET-method. Passes the global variables
 * anno_current_root_id, anno_current_reply_target_id and
 * anno_modification_id as parameter, as well as the error message if given.
 * These are used to show the the view pane and/or error message if needed.
 */
function anno_reload_document() {
    // get the document id, which is actually URL of the document
    // without any Anno specific parameters
    var url = anno_get_document_id();

    // add ? if not present
    if (url.indexOf('?') == -1) {
        url += '?';
    }

    // paste the paremeters to the end of the url if they are not null
    if(anno_current_root_id != null) {
        url += '&anno_current_root_id=' + encodeURIComponent(anno_current_root_id);
    }
    if(anno_current_reply_target_id != null) {
        url += '&anno_current_reply_target_id=' + encodeURIComponent(anno_current_reply_target_id);
    }
    if(anno_modification_target_id != null) {
        url += '&anno_modification_target_id='+ encodeURIComponent(anno_modification_target_id);
    }

    // if in test mode, then just log
    if (anno_test_mode) {
        anno_test_log("\nPage reloaded: url=" + url);
    } else {
        window.location.assign(url);
    }
}


/**
 * Function for logging.
 */
var anno_test_log_str = '';
function anno_test_log(str) {
    document.getElementById('anno_test_mode_output').value += str + "\n"; 
}
