/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/ 
/**
 * Library of functions that handle UI windows. Created to make as litlle 
 *changes as possible to the original javascript and to resole cross browser
 *issues that are missing from the original javascript.
 *
 * @author  Jarno Laitinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */


/**
 * Gives the coordinates of the event
 * 
 * @param e event
 * @return struct coordinates
 */
function anno_util_MouseCoordinates(e) {
    if (navigator.appName.indexOf("Microsoft")!=-1) {
        mX = event.x+document.documentElement.scrollLeft;
        mY = event.y+document.documentElement.scrollTop;
    } else {
        mX = e.pageX;
        mY = e.pageY;
    }
    return { x: mX, y: mY };
}


/**
 * Counts browser windows dimensions
 * 
 * @return struct window dimensions
 */
function anno_get_WindowDimensions(){
    var width = 0, height = 0;
    if (typeof(window.innerWidth) == 'number' ) {
        //Non-IE
        width = window.innerWidth-16;
        height = window.innerHeight;
    } else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) {
        //IE 6+ in 'standards compliant mode'
        width = document.documentElement.clientWidth-20;
        height = document.documentElement.clientHeight;
    } else if (document.body && (document.body.clientWidth || document.body.clientHeight)) {
        //IE 4 compatible
        width = document.body.clientWidth-20;
        height = document.body.clientHeight;
    }
    return { width: width, height: height };
}


/**
 * Gives  comment windows  coordinates
 *
 * @param e event that caused to call this function
 * @param windowName String identifies comment view
 * @return struct coordinates to document
 */
function anno_util_positionWindow(e, windowName) {
    if (parseInt(navigator.appVersion)>3) {
        if (navigator.appName=="Netscape") {
            winW = window.innerWidth;
            if (winW > 16) {
                winW = winW-16;
            }
        }

        if (navigator.appName.indexOf("Microsoft")!=-1) {
            winW =  document.documentElement.clientWidth;
            if (winW > 20) {
                winW = winW-20;
            }
        }
    }
	
    var win = anno_get_WindowDimensions();
    var winH = win.height;
	
    var pos = anno_util_MouseCoordinates(e);
    var x = pos.x; 
    var y = pos.y;
    var windowBottom; 
    if (navigator.appName.indexOf("Microsoft")!=-1) {
        windowBottom = winH+document.documentElement.scrollTop;
    } else{
        windowBottom = winH+window.pageYOffset;
    }

    var obj = anno_util_GetEventElement(e);
    var fs;

    if (navigator.appName.indexOf("Microsoft")!=-1) {
        fs = anno_util_GetCurrentStyle(anno_util_GetEventElement(e), "fontSize");
        y -= 2; 
    }
    if (navigator.appName=="Netscape") {
        fs = anno_util_GetCurrentStyle(anno_util_GetEventElement(e), "font-size");
    }

    var element_dimensions = anno_util_DynamicWidthHeight(e, fs, windowName);
    var commentWidth = element_dimensions.w;
    var commentHeight = element_dimensions.h;

    //Counts elements tops absolute position
    //when new window isn't add new comment window
    if(!(windowName.indexOf("addComment")!=-1)){
        elementTop = anno_util_getAbsolutePosition(obj);
        var elementHeight = obj.offsetHeight;
        var curtop = elementTop.y;

        //Counts elements bottoms absolute position
        //counts elements line height from height NOT MOST ACCURATE
        //because more lines in comment, more this calculation throws off
        //css file be fixed so that you can ask lineheight directly from the element
        var lineHeight=(1+((elementHeight%fs)/fs))*fs;

        //counts next lines position under mouses koordinates
        y = curtop;
        while(y>=curtop) {
            curtop = curtop + lineHeight;
        }

        y = curtop;
    }
	
    //checks that new comment windows stay inside browser
    if (x < 0) {
        x = 0;
    }
    if (commentWidth + x > winW) {
        x = winW-commentWidth;
    }

    if(y - commentHeight < 0){
        y = pos.y - commentHeight;
    }

    if(!(windowName.indexOf("addComment")!=-1)){
        if(y + commentHeight > windowBottom) {
            y = elementTop.y - commentHeight;
        }
    }
    // and again sigled out small add new comment window wich doesn't have a comment/span element to align with
    else{
        if(y + commentHeight > windowBottom) {
            y -= commentHeight;
        }
    }

    return { x: x, y: y };
}


/**
 * Changes css style definition to span element form 
 * unread to read. 
 * span.anno_highlight_spelling_unread 
 * span.anno_highlight_content_unread 
 * span.anno_highlight_spelling_read 
 * span.anno_highlight_content_read
 * 
 * @param el element
 */
function anno_util_changeReadCSS(el){
    el.className=el.className.replace(/_unread/, '_read');
    /*
    if (navigator.appName.indexOf("Microsoft")!=-1) {
        el.setAttribute('className', el.getAttribute('className').replace(/_unread/, '_read'));
    }
    if (navigator.appName=="Netscape") {
        el.setAttribute('class', el.getAttribute('class').replace(/_unread/, '_read'));
    }
    */
}


/**
 * Found this script on this web page
 * http://www.quirksmode.org/js/selected.html
 * This gets same result in every broser as gecko browser window.getSelection
 *
 * @return  oSelection - Object that contains text form the selected area 
 */
function anno_util_getSel() {
    var oSelection; 

    if (window.getSelection){
        oSelection = window.getSelection();
        // Sanity check to make sure there is something selected
        if(oSelection.rangeCount > 0) {
            return window.getSelection().getRangeAt(0);
        } else {
            return null;
        }
    } else if (document.getSelection){
        oSelection = document.getSelection();
    } else if (document.selection){
        oSelection = document.selection.createRange();
    } else {
        return;
    }
    return oSelection;
}


/**
 * Just in case there are different size window. To handle that you just add here new parameter
 * Return window dimensions, which were taken from css file. Since I can't get any style information
 * from window elements I use span elements font size as reference point. Real width in css was 25em
 * But since popup uses smaller font size I have to accommondate this by making multiplier higher-> 31.
 * This approach is complicated but at least it gives illusion of scalebility.
 *
 * @param e event
 * @param fsize size modifier
 * @param windowName name of the window
 * @return struct width and height of the comment view
 */
function anno_util_DynamicWidthHeight(e, fsize, windowName){
    var w;
    var h;
    if (navigator.appName.indexOf("Microsoft")!=-1) {
        if (windowName.indexOf("singleview")!=-1) {
            w = fsize*32;
            h = fsize*13;
        }
        if (windowName.indexOf("multiview")!=-1) {
            w = fsize*32;
            h = fsize*13;
        }
        if (windowName.indexOf("addComment")!=-1) {
            w = fsize*14.5;
            h = fsize*6;
        }
    } else {
        if (windowName.indexOf("singleview")!=-1) {
            w = fsize*26;
            h = fsize*10;
        }
        if (windowName.indexOf("multiview")!=-1) {
            w = fsize*26;
            h = fsize*10;		
        }
        if (windowName.indexOf("addComment")!=-1) {
            w = fsize*12;
            h = fsize*4;		
        }
    }
    return { w: w, h: h };
}


/**
 * Get style information from elemnt. 
 * To get font size
 * Mozilla: font-size
 * IE: fontSize
 *
 * @param el element
 * @param prop style selector
 * @return int style information (used to get font size wich is int that tells font size in px )
 */
function anno_util_GetCurrentStyle(el, prop) {
    if (window.getComputedStyle) {
        // Mozilla et al
        return parseInt(window.getComputedStyle(el, '').getPropertyValue(prop));
    } else if (el.currentStyle) {
        // IE 5+
        var fs;
        fs = parseInt(el.currentStyle[prop]);
        if(fs==100) {
            fs=anno_util_GetCurrentStyle(el.parentNode, prop);
        }
        return fs;
    } else if (el.style) {
        // IE 4
        getCurrentInfo(el);
        return parseInt(el.style[prop]);
    }
}


/**
 * Retrieve the absolute coordinates of an element.
 *
 * @return struct A hash containing keys 'x' and 'y'.
 */
function anno_util_getAbsolutePosition(element) {
    var r = { x: element.offsetLeft, y: element.offsetTop };
    if (element.offsetParent) {
        var tmp = anno_util_getAbsolutePosition(element.offsetParent);
        r.x += tmp.x;
        r.y += tmp.y;
    }
    return r;
}


/**
 * Cross browser solution. 
 * get from this webpage
 * http://www.netlobo.com/javascript_get_element_id.html
 * Finds documents element by id and returns found element
 *
 * @param id id
 * @return  element 
 */
function anno_returnObjById(id) { 
    if (document.getElementById) {
        var returnVar = document.getElementById(id); 
    } else if (document.all) {
        var returnVar = document.all[id]; 
    } else if (document.layers) {
        var returnVar = document.layers[id]; 
    }
    return returnVar; 
} 


/**
 * Disables button by id
 * 
 * @param id id
 */
function anno_disableButton(id){
    var but = anno_returnObjById(id);
    but.focus;
    but.disabled=true;
}


/**
 * Enables button by id
 * 
 * @param id id
 */
function anno_util_enableButton(id){
    var but = anno_returnObjById(id);
    but.focus;
    but.disabled=false;
}


/**
 * Gets events target element
 * 
 * @param event event
 * @return target element 
 */
function anno_util_GetEventElement(event) {
    if (!event) {
        var event = window.event;
    }
    var target = event.target ? event.target : event.srcElement;
    return target;	
}


/**
 * Creates pop-up useful for testing
 */
/*
function anno_util_printNewWindow(text) {
    var generator=window.open("", "xpointer", "height=200,width=1200,status=yes,toolbar=no,menubar=no,location=no");

    //var generator=window.open('about:blank','name','height=200,width=1200');
    generator.focus();
    //window.setTimeout("generator.focus()", 000);  // This will execute, but afterwards the window that executes
                                                    // it will reload and come on top of the popup window
	
    generator.document.write('<html><head><title>Popup</title>');
    generator.document.write('<link rel="stylesheet" href="style.css">');
    generator.document.write('</head><body>');
    generator.document.write('<p>');
    generator.document.write(text);
    generator.document.write('<p><a href="javascript:self.close()"> Close</a> the popup.</p>');
    generator.document.write('</body></html>');
    generator.document.close();
}
*/
