/*  Anno - a system for interoperation of Moodle with an Annotea server.
    Anno is a modified version of Mooan.
    Anno  Copyright (C) 2007 Jukka Huhta, Juho Iso-Markku, Jarno Laitinen,
                        Timo Myyrylinen, Roger Sandstrm, Miro Wikgren
    Mooan Copyright (C) 2006  Heikki Aitakangas, Mika Holmstrm,
                        Kirsi Jokisalo, Anni Kotilainen, Esko Lehtonen,
                        Paula Urpilainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
/**
 * Functions to use XPointers.
 *
 * @author  ??, Jarno Laitinen
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 *
 * Created by mooan-group August 2006 for the Mooan-software project and
 * modified by anno3-group May 2007 for the Anno-software project
 * at the Department of Computer science, University of Helsinki.
 *
 */


/**
 * Generates an XPointer from the current selection, using the target element of the
 * mouseup event that signified the user finishing the selection to locate the selection.
 *
 * This is inaccurate, if the content of the selection is repeated within the first
 * element to contain the whole selection, the seek algorithm will always select the
 * first repetition.
 * This could be reduced to failing only if the whole selection is contained within
 * one #text node and is repeated inside that node. However, that requires getting
 * from the browser the exact element from which the selection begins (or the exact
 * element in which the selection ends, but in that case the seek algorithm becomes
 * somewhat more evil) and we have not figured out how to do so yet.
 *
 * This is mostly a copy of the Anno-project's AnnoDocument_sGetCurrentXPointer()
 * function, with some minor code changes and lots of added comments.
 *
 *
 * @param eOnMouseUpTargetElement The element that is the target of the OnMouseUp event
 *                                caused by the user finishing mouse-dragging to generate
 *                                a selection. Hopefully not too far from the element in
 *                                which the selection actually ends.
 *
 * @return A string in the form of url#xpointer or null in case of an empty selection
 */
function sGetXPointerFromCurrentSelection(eOnMouseUpTargetElement) {
    
    //these buttons will be disabled later if selection overlaps on spelling error or if there is exactly the same contects annotation allready
    anno_util_enableButton("anno_oikeinkirjoitus_nappi");
    anno_util_enableButton("anno_sisaltonappi");
	
    // Get the selection object
    var oSelectionRange = oGetSelectionRange();
	
    // Make sure it's a non-empty selection
    if (!oSelectionRange) {
        return null;
    }
    if (oSelectionRange.toString) {
        if (oSelectionRange.toString().length==0) { // empty selection in Firefox
            return null;
        }
    } else { 
        if (oSelectionRange.text.length==0) { // empty selection in IE
            return null;
        }
    } 
	
    // Create the variables that are missing from IE: 
    if (oSelectionRange.parentElement) { // check for IE compatibility
        var startRange=oSelectionRange.duplicate();
        startRange.moveEnd("character",-(startRange.text.length-1)); // take the first character of the selection
        oSelectionRange.startContainer=startRange.parentElement();  // find out which element it is in
        var endRange=oSelectionRange.duplicate();
        endRange.moveStart("character",(endRange.text.length-1)); // take the last character
        oSelectionRange.endContainer= endRange.parentElement();  // find out which element it is in
  
        // find the starting offset for the selection: 
        var tmpRange =document.body.createTextRange();
        tmpRange.moveToElementText(startRange.parentElement()); // create a Range that covers the whole element where the range starts
        var i=0; 
        while (tmpRange.compareEndPoints("EndToStart",startRange)==1) { // compare the starting points of the ranges
            tmpRange.moveEnd("character",-1);  // move the end point of the range by 1 char
        } 
  
        var precidingText=tmpRange.text; 
        oSelectionRange.startOffset=precidingText.length; // now we should have the starting offset!

        // find the end offset for the selection:
        var tmpRange =document.body.createTextRange();
        tmpRange.moveToElementText(endRange.parentElement()); // create a Range that covers the whole element where the range ends
        i=0; 
        while (tmpRange.compareEndPoints("EndToEnd",endRange)==1) { // compare the starting points of the ranges
            tmpRange.moveEnd("character",-1);  // move the end point of the range by 1 char 
        } 
        var precidingText=tmpRange.text; 
        oSelectionRange.endOffset=precidingText.length; // now we should have the starting offset!

        // This is a workaround for a bug in IE; if the oSelectionRange contains '<br>' tags (newlines),
        // the comapreEndPoints()-function works incorrectly making the text too long
        var tmpStr = new Array();
        tmpStr = oSelectionRange.text.split('\n');
        if (tmpStr.length>2) { // more than 1 newline
            oSelectionRange.startOffset+=tmpStr.length-2;
            oSelectionRange.endOffset-=tmpStr.length-2;
        }
    } 

    // Get the start and end elements. If they are #text, instead get their parents 
    var eStartElement = oSelectionRange.startContainer.nodeName == '#text' ? 
                         oSelectionRange.startContainer.parentNode :
                         oSelectionRange.startContainer; 
    var eEndElement   = oSelectionRange.endContainer.nodeName == '#text' ? 
                         oSelectionRange.endContainer.parentNode :
                         oSelectionRange.endContainer;
    var iStartOffset = oSelectionRange.startOffset; 
    var iEndOffset   = oSelectionRange.endOffset;

    if (!oSelectionRange.parentElement) { // This is a workaround for a bug in Firefox
        if (oSelectionRange.startContainer.nodeName != '#text') { // it's a bug in Firefox! 
            var tmpNode=oSelectionRange.startContainer.firstChild;
            var i=0;
            while (i<iStartOffset) { // iStartOffset actually tells the number of the sibling we want
                tmpNode=tmpNode.nextSibling;
                i++;
            }
            var textNode=findNextTextNode(tmpNode);  // search until we find a #text node (type=3)
            tmpNode=textNode; 
            oSelectionRange.setStart(tmpNode,0); 
            iStartOffset = oSelectionRange.startOffset; 
        } 
                          
        if (oSelectionRange.endContainer.nodeName != '#text') { // it's a bug in Firefox!
            var tmpNode=oSelectionRange.endContainer.firstChild;
            var i=0; 
            while (i<iEndOffset) { // iEndOffset actually tells the number of the sibling we want
                tmpNode=tmpNode.nextSibling; 
                i++;
            } 
            var textNode=findNextTextNode(tmpNode); // search until we find a #text node (type=3)
            tmpNode=textNode; 
            oSelectionRange.setEnd(tmpNode,0);
            iEndOffset   = oSelectionRange.endOffset;
        } 
    } 

    var eStartElement = oSelectionRange.startContainer.nodeName == '#text' ?
                         oSelectionRange.startContainer.parentNode : 
                         oSelectionRange.startContainer; 
    var eEndElement   = oSelectionRange.endContainer.nodeName == '#text' ?
                         oSelectionRange.endContainer.parentNode :
                         oSelectionRange.endContainer; 

    // If the start container was #text and not it's parent's first child, seek the
    // offset of the start point within the parent node by summing the lengths of
    // preceding sibling #text nodes to the offset
    if (oSelectionRange.startContainer.nodeName == '#text' && oSelectionRange.startContainer.previousSibling) {
        var stack = new Array();
        stack.push(oSelectionRange.startContainer.parentNode);
        var currentNode;

        // Loop through the subtree starting at the start container's parent
        // until we either run out of nodes (shouldn't happen) or we encounter the start container
        while(stack.length > 0 && (currentNode = stack.pop()) != oSelectionRange.startContainer) {
            // If it's a #text node, add it's content's length to the sum
            if (currentNode.nodeName == '#text') {
                iStartOffset += currentNode.nodeValue.length;
            }
            if(currentNode.nodeName == "span") {
                bIsIgnorableNode(currentNode.nodeName);
            }
            // Push the node's children into the stack in reverse order
            if (currentNode.hasChildNodes()) {
                for (var i = currentNode.childNodes.length - 1; i >= 0; i--) {
                    stack.push(currentNode.childNodes[i]);
                }
            }
        }
    }

    // Same for the end node
    if (oSelectionRange.endContainer.nodeName == '#text' && oSelectionRange.endContainer.previousSibling) {
        var stack = new Array();
        stack.push(oSelectionRange.endContainer.parentNode);
        var currentNode;

        // Loop through the subtree starting at the end container's parent
        // until we either run out of nodes (shouldn't happen) or we encounter the end container
        while(stack.length > 0 && (currentNode = stack.pop()) != oSelectionRange.endContainer) {
            // If it's a #text node, add it's content's length to the sum
            if (currentNode.nodeName == '#text') {
                iEndOffset += currentNode.nodeValue.length;
            }
            if(currentNode.nodeName == "span") {
                bIsIgnorableNode(currentNode.nodeName);
            }

            // Push the node's children into the stack in reverse order
            if (currentNode.hasChildNodes()) {
                for (var i = currentNode.childNodes.length - 1; i >= 0; i--) {
                    stack.push(currentNode.childNodes[i]);
                }
            }
        }
    }

    // Build the XPointer
    var sXPointer = anno_get_document_id() + '#xpointer(';
         
    // Simpler XPointer if the whole selection is contained within one element
    if (eStartElement == eEndElement) {
        // url#xpointer(string-range(xpath, "", startindex, length)) 
        var tmp= sCreateXPathAdjustOffset(eStartElement); 
        iStartOffset=iStartOffset+tmp.offset; 
        iEndOffset=iEndOffset+tmp.offset; 
        sXPointer += 'string-range(' + tmp.path 
                                     + ', "", '
                                     + iStartOffset + ', '
                                     + (iEndOffset - iStartOffset) 
                                     + ')';      
    } else {
        // Different elements, need to use range-to
        // url#xpointer(start-point(string-range(..))/range-to(end-point(string-range(..))))
        var tmp= sCreateXPathAdjustOffset(eStartElement);
        iStartOffset=iStartOffset+tmp.offset;
         
        var tmp2= sCreateXPathAdjustOffset(eEndElement);
        iEndOffset=iEndOffset+tmp2.offset;
     
        if (tmp.path==tmp2.path) { // check whether start and end are within the same element, after we have filtered out ignorable nodes
            sXPointer += 'string-range(' + tmp.path
                                         + ', "", '
                                         + iStartOffset + ', ' 
                                         + (iEndOffset - iStartOffset)
                                         + ')';  
        } else { 
            sXPointer += 'start-point('; 
            sXPointer += 'string-range(' + tmp.path + ', "", ' + iStartOffset + ', 1)';
            sXPointer += ')/range-to(end-point('; 
            sXPointer += 'string-range(' + tmp2.path   + ', "", ' + iEndOffset   + ', 1)))';
        } 
    } 
    
    sXPointer += ')'; 
    anno_checkElements(sXPointer, eStartElement);
	
    return sXPointer;
}


/**  
 * Finds the first #text node after the current node. 
 *
 * @param element element
 * @return The first #text node found, or null.     
 */ 
function findNextTextNode(element) { 
    var tmp; 
    if (element.nodeType!=1) {
        // #text node
        return element;
    }
    if (element.firstChild) { 
        // search the first child, recursively
        tmp=findNextTextNode(element.firstChild);
        if (tmp) return tmp;
    } 
    if (element.nextSibling) {
        // search the next sibling, recursively 
        tmp=findNextTextNode(element.nextSibling);  
        if (tmp) return tmp; 
    } 
    return null;   
} 


/**
 * Tries to retrieve a domrange object describing the selection.
 * Currently supported browser engines:
 *  - Gecko
 *
 * @return The selection object or null if one couldn't be found
 */
function oGetSelectionRange() {
    var oSelection = anno_util_getSel();
    return oSelection;
}


/**
 * Determines whether the given DOM node should be ignored
 * either due to being added by Anno.
 * 
 * @param object node
 * @return boolean true if node is ignorable
 */
function bIsIgnorableNode(node) {
    // Check for nodes addedby Anno
    if (node.attributes) {
        var nodeID = node.getAttribute('id');
        if (nodeID != null && nodeID.substring(0, 5) == 'anno_') {
            //this prevents to mark spelling errors on top of each other
            return true;
        }
    }
    return false;
}


/**
 * Creates an XPath to a given element.
 * If the given element is not valid,
 * returns an empty string.
 *
 * @param element element
 * @return string xpath
 */
function sCreateXPath(element) {
    // Loop up to root node of the document
    for (var sXPath=''; element && element.nodeName != '#document'; element = element.parentNode ) {
        if (element.nodeName != '#text' && !bIsIgnorableNode(element)) {
            // If not text node, find out if previous siblings with same element name
            // Loop through all previous siblings and count elements with same name
            for (var iElementIndex = 1, sibling = element.previousSibling; sibling; sibling = sibling.previousSibling) {
                if (sibling.nodeName == element.nodeName){
                    iElementIndex++;
                }
            }

            // Add element[iElementIndex] to sXPath expression
            sXPath = '/' + element.nodeName.toLowerCase() + '[' + iElementIndex + ']' + sXPath;
        }
    }

    // If we did generate something above, add in the other leading slash
    if (sXPath != '') {
        sXPath = '/' + sXPath;
    }
    return sXPath;
}


/** 
 * Creates an XPath to a given element, and adjusts the offset when there are ignorable nodes in the path
 * If the given element is not valid, 
 * returns an empty string. 
 *
 * @param element element 
 * @return string xpath
 */ 
function sCreateXPathAdjustOffset(element) {
    var offset=0;
    // Loop up to root node of the document
    for (var sXPath=''; element && element.nodeName != '#document'; element = element.parentNode ) {
        if (element.nodeName != '#text' && !bIsIgnorableNode(element)) {
            // If not text node, find out if previous siblings with same element name
            // Loop through all previous siblings and count elements with same name
            for (var iElementIndex = 1, sibling = element.previousSibling; sibling; sibling = sibling.previousSibling) {
                if (sibling.nodeName == element.nodeName){
                    iElementIndex++;
                } 
            } 
     
            // Add element[iElementIndex] to sXPath expression 
            sXPath = '/' + element.nodeName.toLowerCase() + '[' + iElementIndex + ']' + sXPath;
        } else if (bIsIgnorableNode(element)) { 
          offset=offset + getOffset(element); 
        } 
    }    

    // If we did generate something above, add in the other leading slash
    if (sXPath != '') {
        sXPath = '/' + sXPath; 
    } 
     
    return  {path: sXPath, offset: offset }; 
} 


/** 
 * Returns the starting offset of the given element within its parent element 
 * 
 * @param element element
 * @return int offset
 */  
function getOffset(element) { 
    var i=0; 
    if (navigator.appName.indexOf("Microsoft")!=-1) {
        // find the starting offset for the selection: 
        var range1 =document.body.createTextRange(); 
        range1.moveToElementText(element); // create a Range that covers the whole element
        var range2 =document.body.createTextRange();
        range2.moveToElementText(element.parentNode); // create a Range that covers the parent
      
        while (range2.compareEndPoints("StartToStart",range1)==-1) { // compare the starting points of the ranges
            range2.moveStart("character",1);  // move the starting point of the range by 1 char
            i=i+1;  // calculate how many chars we have moved
        } 
        return i; // now we have the offset!
    } else {
        while (element=element.previousSibling) {
            i=i+element.textContent.length;
        }
        return i;
    } 
}


/**
 * This Function uses  two dimensional array commentarray, where are all
 * old xpointers stored with comment information wich tells weather comment is
 * content type or spelling type. This array is writen by php to html pages source
 * under header tag. 
 * 
 * Checks are made fore every selection.
 * If selection is over content type comment  and if the selection is exactly same 
 * selection as that content type comment. Then  function prevents new content type 
 * comment by disablying that button from add comment  window. 
 *
 * For spelling comment check it goes through selection and if there is span element
 * with name spelling then button will be disabled
 * 
 * @param sXPointer - New selections xpointer
 * @param eStartElement -  New Selections starting element
 */
function anno_checkElements(sXPointer, eStartElement){
    var temp;
    var text;
    temp = sXPointer.split("#");
    sXPointer = temp[1];
    var noComments = true;
    try {
        if(commentarray != null){
            noComments=false;
        }
    }catch (e){ 
        //there are no comments in document
        noComments=true;
    }
    if(!noComments){
        for(var i=0; i<commentarray.length; i++){
            //disable content button
            if(commentarray[i][1]=="content"){
                temp = commentarray[i][0].split("#");
                xpoint = temp[1];
                if(sXPointer.indexOf(xpoint) != -1){
                    anno_disableButton("anno_sisaltonappi");
                }
            //disable spelling button 
            }
        }
    
        var checkSelection;
        var spans = anno_returnObjById("content").getElementsByTagName("span");
        if (window.getSelection) { //mozilla
            checkSelection=window.getSelection();
            for(var i=0; i<spans.length; i++){
                if(checkSelection.containsNode(spans[i], true) && (spans[i].className).indexOf("spelling")!=-1){
                    anno_disableButton("anno_oikeinkirjoitus_nappi");
                }
            }
        }
        if (document.selection){ //IE
            checkSelection = document.selection.createRange().htmlText;
            //checks if there is spelling span element in selection or in selections parent element 
            //because content and spelling span elements are allowed to go on top of each other
            if(checkSelection.indexOf("anno_highlight_spelling") !=-1 || (eStartElement.className).indexOf("anno_highlight_spelling") !=-1 || (eStartElement.parentNode.className).indexOf("anno_highlight_spelling") !=-1){
                anno_disableButton("anno_oikeinkirjoitus_nappi");
            }
        }
    }
}	

